#!/bin/csh -f
# C-shell script to run the program unres (serial version);
# by A. Liwo, last revision 9/15/99
#
echo "This job is running on host `hostname` "
if ($#argv<2) then
  echo "You must supply potential type and input/output filename prefix."
  exit (1)
endif

# Put YOUR UNRES root directory name here.
set UNRES_ROOT = /people2/adam/PROTARCH
# Here, following directories are defined relative to the root directory
# specified above.
set DD = ${UNRES_ROOT}/PARAM
set BIN = ${UNRES_ROOT}/UNRES/bin
set SRC = ${UNRES_ROOT}/UNRES/src

set POT = $1
setenv POT $1
setenv PREFIX $2
#
# Is the specified potential in the list of valid force fields?
#
if (($POT != "LJ") && ($POT != "LJK") && ($POT != "BP") && ($POT != "GB") \
&& ($POT != "GBV") ) then
  echo "Unknown potential type $POT"
  exit(1)
endif

# Check for the existence of the input file

if (!(-e $2.inp)) then
  echo "Error: The input file $2.inp does not exist."
  exit(1)
endif

# 5/17/97 parameter file assignment was changed from link to SETENV statements
#
setenv THETPAR $DD/thetaml.5parm
setenv ROTPAR $DD/scgauss.parm
setenv TORPAR $DD/torsion_cryst.parm
setenv ELEPAR $DD/electr.parm
setenv SIDEPAR $DD/scinter_$1.parm

# Pattern database
setenv PATTERN $DD/patterns.cart

# Change following line to set debug = 1, if you want a debug run
set debug = 0
if (${debug} == 1) then
# following line for AIX
#   dbx -I ${SRC}/src -I ${SRC}/f_FILES ${BIN}/unres
# following line for LINUX
   gdb -d ${SRC}/src -d ${SRC}/f_FILES ${BIN}/unres
else
   time $BIN/unres
endif

# Remove links
/bin/rm fort.* 

# Clear empty files
find . -size 0 -exec rm {} \;
