#!/bin/csh -f
# Script to run the program unres (serial version).
#
echo "This job is running on host `hostname` "
if ($#argv<2) then
  echo "You must supply potential type and input/output filename prefix."
  exit (1)
endif

# Change these two lines to match the directories where you put UNRES/CSA
set UNRES_ROOT = ~/work/PROTARCH
set DD = ${UNRES_ROOT}/PARAM
set BIN = ${UNRES_ROOT}/UNRES/bin

set POT = $1
setenv POT $1
setenv PREFIX $2
#
# Is the specified potential in the list of valid force fields?
#
if (($POT != "LJ") && ($POT != "LJK") && ($POT != "BP") && ($POT != "GB") \
&& ($POT != "GBV") ) then
  echo "Unknown potential type $POT"
  exit(1)
endif

# Check for the existence of the input file

if (!(-e $2.inp)) then
  echo "Error: The input file $2.inp does not exist."
  exit(1)
endif

# 5/17/97 parameter file assignment was changed from link to SETENV statements
#
setenv THETPAR $DD/thetaml.5parm
setenv ROTPAR $DD/scgauss.parm
setenv TORPAR $DD/torsion_cryst.parm
setenv ELEPAR $DD/electr.parm
setenv SIDEPAR $DD/scinter_$1.parm

# Pattern database
setenv PATTERN $DD/patterns.cart

# Set parallel environment, depending on job type (interactive or batch).

if ($?LL_JOB) then
  setenv MP_HOSTFILE NULL
  setenv MP_EUILIB us
  setenv MP_EUIDEVICE css0 
else 
  setenv MP_RMPOOL 0
  setenv MP_RESD yes
  setenv MP_HOSTFILE NULL
  setenv MP_EUILIB ip
  setenv MP_EUIDEVICE css0
endif

setenv MP_PROCS $3
setenv FGPROCS $4

ln -s -f ${PREFIX}.CSA.out fort.10
ln -s -f ${PREFIX}.CSA.min fort.11
ln -s -f ${PREFIX}.CSA.angl fort.12
ln -s -f ${PREFIX}.CSA.index fort.13
ln -s -f ${PREFIX}.CSA.which fort.14
ln -s -f ${PREFIX}.CSA.start fort.19
ln -s -f ${PREFIX}.CSA.in fort.40
ln -s -f ${PREFIX}.CSA.rbank fort.30
ln -s -f ${PREFIX}.CSA.seed fort.31
ln -s -f ${PREFIX}.CSA.history fort.32
ln -s -f ${PREFIX}.CSA.bank fort.33
ln -s -f ${PREFIX}.CSA.bank1 fort.34
ln -s -f ${PREFIX}.CSA.alpha fort.35
ln -s -f ${PREFIX}.CSA.alpha1 fort.36

set DEBUG = 0

if (${DEBUG} == 1) then
  pdbx -I ~/UNRES/NEW/src -I ~/UNRES/NEW/src/f_FILES $BIN/unresCSA
else if ($?LL_JOB) then
  $BIN/unresCSA
else
  poe $BIN/unresCSA
endif

# Remove links
/bin/rm fort.* 

# Clear empty files
find . -size 0 -exec rm {} \;
