#!/bin/csh -f
# Script to run the program unresCSA on linux with MPICH (parallel version).
#

setenv env $HOME/.env
rm -f $env
touch $env

echo "This job is running on host `hostname` "
if ($#argv<2) then
  echo "You must supply potential type and input/output filename prefix."
  exit (1)
endif

set UNRES_ROOT = /people2/adam/PROTARCH
set DD = ${UNRES_ROOT}/PARAM
echo "setenv DD ${UNRES_ROOT}/PARAM " > $env
set BIN = ${UNRES_ROOT}/UNRES/bin
echo "setenv BIN ${UNRES_ROOT}/UNRES/bin " > $env

set POT = $1
setenv POT $1
echo "setenv POT $1 " >> $env

setenv PREFIX $2
echo "setenv PREFIX $2 " >> $env

#
# Is the specified potential in the list of valid force fields?
#
if (($POT != "LJ") && ($POT != "LJK") && ($POT != "BP") && ($POT != "GB") \
&& ($POT != "GBV") ) then
  echo "Unknown potential type $POT"
  exit(1)
endif

# Check for the existence of the input file

if (!(-e $2.inp)) then
  echo "Error: The input file $2.inp does not exist."
  exit(1)
endif

# 5/17/97 parameter file assignment was changed from link to SETENV statements
#
setenv THETPAR $DD/thetaml.5parm
echo "setenv THETPAR $DD/thetaml.5parm " >> $env
setenv ROTPAR $DD/scgauss.parm
echo "setenv ROTPAR $DD/scgauss.parm " >> $env
setenv TORPAR $DD/torsion_cryst.parm
echo "setenv TORPAR $DD/torsion_cryst.parm " >> $env
setenv ELEPAR $DD/electr.parm
echo "setenv ELEPAR $DD/electr.parm " >> $env
setenv SIDEPAR $DD/scinter_$1.parm
echo "setenv SIDEPAR $DD/scinter_$1.parm  " >> $env

# Pattern database
setenv PATTERN $DD/patterns.cart
echo "setenv PATTERN $DD/patterns.cart " >> $env

# Set parallel environment, depending on job type (interactive or batch).

setenv MP_PROCS $3
setenv FGPROCS $4
echo "setenv FGPROCS $4 " >> $env

ln -s -f ${PREFIX}.CSA.out fort.10
ln -s -f ${PREFIX}.CSA.min fort.11
ln -s -f ${PREFIX}.CSA.angl fort.12
ln -s -f ${PREFIX}.CSA.index fort.13
ln -s -f ${PREFIX}.CSA.which fort.14
ln -s -f ${PREFIX}.CSA.start fort.19
ln -s -f ${PREFIX}.CSA.in fort.40
ln -s -f ${PREFIX}.CSA.rbank fort.30
ln -s -f ${PREFIX}.CSA.seed fort.31
ln -s -f ${PREFIX}.CSA.history fort.32
ln -s -f ${PREFIX}.CSA.bank fort.33
ln -s -f ${PREFIX}.CSA.bank1 fort.34
ln -s -f ${PREFIX}.CSA.alpha fort.35
ln -s -f ${PREFIX}.CSA.alpha1 fort.36

set DEBUG = 0

limit coredumpsize 0
time mpirun -np $3 $BIN/unresCSA
# rm -f $env
# touch $env


# Remove links
/bin/rm fort.* 

# Clear empty files
find . -size 0 -exec rm {} \;
