      subroutine mcm_setup
      include 'DIMENSIONS'
      include 'COMMON.CACHE'
      include 'COMMON.IOUNITS'
      include 'COMMON.MCM'
      include 'COMMON.CONTROL'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
C
C Set up variables used in MC/MCM.
C    
      write (iout,'(80(1h*)/20x,a/80(1h*))') 'MCM control parameters:'
      write (iout,'(5(a,i7))') 'Maxacc:',maxacc,' MaxTrial:',MaxTrial,
     & ' MaxRepm:',MaxRepm,' MaxGen:',MaxGen,' MaxOverlap:',MaxOverlap
      write (iout,'(4(a,f8.1)/2(a,i3))') 
     & 'Tmin:',Tmin,' Tmax:',Tmax,' TstepH:',TstepH,
     & ' TstepC:',TstepC,'NstepH:',NstepH,' NstepC:',NstepC 
      if (nwindow.gt.0) then
        write (iout,'(a)') 'Perturbation windows:'
        do i=1,nwindow
          i1=winstart(i)
          i2=winend(i)
          it1=itype(i1)
          it2=itype(i2)
          write (iout,'(a,i3,a,i3,a,i3)') restyp(it1),i1,restyp(it2),i2,
     &                        ' length',winlen(i)
        enddo
      endif
C Rbolt=8.3143D-3*2.388459D-01 kcal/(mol*K)
      RBol=1.9858D-3
C Number of "end bonds".
      koniecl=0
c     koniecl=nphi
      print *,'koniecl=',koniecl
C Probabilities of different move types
      sumpro_type(0)=0.0D0
c     sumpro_type(1)=1.0D0
      sumpro_type(1)=0.0D0
      sumpro_type(2)=1.3D0
c     sumpro_type(3)=1.8D0      
      sumpro_type(3)=1.3D0      
c     sumpro_type(4)=2.3D0
      sumpro_type(4)=1.3D0      
      if (minim) then
        sumpro_type(3)=sumpro_type(2)
        sumpro_type(4)=sumpro_type(3)+0.5D0
      endif
C Maximum length of N-bond segment to be moved
c     nbm=nres-1-(2*koniecl-1)
      if (nwindow.gt.0) then
        maxwinlen=winlen(1)
        do i=2,nwindow
          if (winlen(i).gt.maxwinlen) maxwinlen=winlen(i)
        enddo
        nbm=min0(maxwinlen,6)
        write (iout,'(a,i3,a,i3)') 'Nbm=',Nbm,' Maxwinlen=',Maxwinlen
      else
        nbm=min0(6,nres-2)
      endif
      sumpro_bond(0)=0.0D0
      sumpro_bond(1)=0.0D0 
      do i=2,nbm
        sumpro_bond(i)=sumpro_bond(i-1)+1.0D0/dfloat(i)
      enddo
      write (iout,'(a)') 'The SumPro_Bond array:'
      write (iout,'(8f10.5)') (sumpro_bond(i),i=1,nbm)
C Maximum number of side chains moved simultaneously
c     print *,'nnt=',nnt,' nct=',nct
      ngly=0
      do i=nnt,nct
        if (itype(i).eq.10) ngly=ngly+1
      enddo
      mmm=nct-nnt-ngly+1
      if (mmm.gt.0) then
        MaxSideMove=min0((nct-nnt+1)/2,mmm)
      endif
c     print *,'MaxSideMove=',MaxSideMove
C Max. number of generated confs (not used at present).
      maxgen=10000
C Set initial temperature
      Tcur=Tmin
      betbol=1.0D0/(Rbol*Tcur)
      write (iout,'(a,f8.1,a,f10.5)') 'Initial temperature:',Tcur,
     &    ' BetBol:',betbol
      write (iout,*) 'RanFract=',ranfract
      return
      end
c------------------------------------------------------------------------------
      subroutine do_mcm(i_orig,iprint)
C Monte-Carlo-with-Minimization calculations - parallel code.
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.CHAIN'
      include 'COMMON.MCM'
      include 'COMMON.CONTACTS'
      include 'COMMON.CONTROL'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.INFO'
      include 'COMMON.CACHE'
      include 'COMMON.DEFORM'
      include 'COMMON.DEFORM1'
      logical accepted,over,ovrtim,error,lprint,not_done,my_conf,
     &        enelower,non_conv
      integer MoveType,nbond,conf_comp
      integer ifeed(max_cache)
      double precision varia(maxvar),varold(maxvar),elowest,eold,
     & przes(3),obrot(3,3)

C---------------------------------------------------------------------------
C Initialize counters.
C---------------------------------------------------------------------------
C Total number of generated confs.
      ngen=0
C Total number of moves. In general this won't be equal to the number of
C attempted moves, because we may want to reject some "bad" confs just by
C overlap check.
      nmove=0
C Total number of temperature jumps.
      ntherm=0
C Total number of shift (nbond_move(1)), spike, crankshaft, three-bond,...
C motions.
      ncache=0
      do i=1,nres
        nbond_move(i)=0
      enddo
C Initialize total and accepted number of moves of various kind.
      do i=0,MaxMoveType
        moves(i)=0
        moves_acc(i)=0
      enddo
C Total number of energy evaluations.
      neneval=0
      nfun=0
      nsave=0
#ifdef MPL
      if (MyID.eq.MasterID) then
        do i=1,nctasks
          nsave_part(i)=0
        enddo
      endif
#endif
      write (iout,*) 'RanFract=',RanFract
#ifndef MPL
      WhatsUp=0
      Kwita=0
#endif
c----------------------------------------------------------------------------
C Compute and print initial energies.
c----------------------------------------------------------------------------
      call intout
      write (iout,'(/80(1h*)/a)') 'Initial energies:'
      call chainbuild
      nf=0
#ifdef MPL
      if (fgProcs.gt.1) then
C Request energy computation from slave processors.
        call geom_to_var(nvar,varia)
        varia(nvar+1)=nf
        varia(nvar+2)=-1
        varia(nvar+3)=icg
        call mp_bcast(varia(1),msglen_var,BossID,fgGroupID)
      endif
#endif
      call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,edihcnstr,
     &  ecorr,etot)
      call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                     edihcnstr,ecorr,etot)
C Minimize the energy of the first conformation.
      if (minim) then
        call geom_to_var(nvar,varia)
!       write (iout,*) 'The VARIA array'       
!       write (iout,'(8f10.4)') (rad2deg*varia(i),i=1,nvar)
        call minimize(etot,varia,iretcode,nfun)
        call var_to_geom(nvar,varia)
        call chainbuild
        write (iout,*) 'etot from MINIMIZE:',etot
!       write (iout,*) 'Tha VARIA array'       
!       write (iout,'(8f10.4)') (rad2deg*varia(i),i=1,nvar)
#ifdef MPL
        if (fgProcs.gt.1) then
C Request energy computation from slave processors.
          call geom_to_var(nvar,varia)
          varia(nvar+1)=nf
          varia(nvar+2)=-1
          varia(nvar+3)=icg
          call mp_bcast(varia(1),msglen_var,BossID,fgGroupID)
        endif
#endif
        call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &    edihcnstr,ecorr,etot)
        write (iout,'(/80(1h*)/a/80(1h*))') 
     &    'Results of the first energy minimization:'
        call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                     edihcnstr,ecorr,etot)
      endif
      if (refstr) then
        call fitsq(rms,c(1,nstart_seq),cref(1,nstart_sup),nsup,przes,
     &             obrot,non_conv)
        rms=dsqrt(rms)
        call contact(.false.,ncont,icont)
        frac=contact_fract(ncont,ncont_ref,icont,icont_ref)
        write (iout,'(a,f8.3,a,f8.3)') 
     &    'RMS deviation from the reference structure:',rms,
     &    ' % of native contacts:',frac*100
        if (print_stat)
     &  write (istat,'(i5,10(1pe14.5))') 0,
     &   evdw,evdw2,evdw1+ees,ecorr,ebe,escloc,etors,etot,rms,frac
      else
        if (print_stat) write (istat,'(i5,9(1pe14.5))') 0,
     &   evdw,evdw2,evdw1+ees,ecorr,ebe,escloc,etors,etot
      endif
      if (print_stat) close(istat)
      neneval=neneval+nfun+1
      write (iout,'(/80(1h*)/20x,a/80(1h*))')
     &    'Enter Monte Carlo procedure.'
      if (print_int) then
        close(igeom)
        call briefout(0,etot)
      endif
      eold=etot
      do i=1,nvar
        varold(i)=varia(i)
      enddo
      elowest=etot
      call zapis(varia,etot)
      nacc=0         ! total # of accepted confs of the current processor.
      nacc_tot=0     ! total # of accepted confs of all processors.
#ifdef MPL
      call recv_stop_sig(Kwita)
      if (whatsup.eq.1) then
        call send_stop_sig(-2)
        not_done=.false.
      else if (whatsup.le.-2) then
        not_done=.false.
      else if (whatsup.eq.2) then
        not_done=.false.
      else
        not_done=.true.
      endif
      if (MyID.eq.MasterID) then
!        ncache=0
        write (iout,*) 'First call to add2cache - update XCACHE.'
        call add2cache(maxvar,max_cache,ncache,nvar,nphi,MasterID,
     &     CachSrc,isent,iused,1,etot,varia,cache_update,ecache,xcache)
        write (iout,*) 'Exit first ADD2CACHE ncache=',ncache
        call receive_MCM_info
        call receive_conf
!        cache_update=.true.
      else
        call send_MCM_info(2)
        call send_conf(nvar,etot,varia,MasterID)
!        ncache=0
        write (iout,*) 'First call to add2cache - update XCACHE.'
        call add2cache(maxvar,max_cache,ncache,nvar,nphi,MyID,
     &    CachSrc,isent,iused,1,etot,varia,cache_update,ecache,xcache)
        write (iout,*) 'Exit first ADD2CACHE ncache=',ncache
        call receive_conf
      endif
#else
      not_done = (iretcode.ne.11)
#endif
C----------------------------------------------------------------------------
C Main loop.
c----------------------------------------------------------------------------
      it=0
      nout=0
      do while (not_done)
        it=it+1
        write (iout,'(80(1h*)/20x,a,i7)')
     &                             'Beginning iteration #',it
C Initialize local counter.
        ntrial=0 ! # of generated non-overlapping confs.
        noverlap=0 ! # of overlapping confs.
        accepted=.false.
        do while (.not. accepted)
#ifdef MPL
          if (Kwita.lt.0) call recv_stop_sig(Kwita)
          if (MyID.eq.MasterID) then
            call receive_MCM_info
            call receive_conf
          endif
#endif
C Retrieve the angles of previously accepted conformation
          do j=1,nvar
            varia(j)=varold(j)
          enddo
          call var_to_geom(nvar,varia)
C Rebuild the chain.
          call chainbuild
C Heat up the system, if necessary.
          call heat(over)
C If temperature cannot be further increased, stop.
          if (over) goto 20
          MoveType=0
          nbond=0
          lprint=.true.
cd        write (iout,'(a)') 'Old variables:'
cd        write (iout,'(10f8.1)') (rad2deg*varia(i),i=1,nvar)
C Decide whether to generate a random conformation or perturb the old one
          RandOrPert=ran_number(0.0D0,1.0D0)
          if (RandOrPert.gt.RanFract) then
            if (print_mc.gt.0)
     &        write (iout,'(a)') 'Perturbation-generated conformation.'
            call perturb(error,lprint,MoveType,nbond,1.0D0)
            if (error) goto 20
            if (MoveType.lt.1 .or. MoveType.gt.MaxMoveType) then
              write (iout,'(/a,i7,a/)') 'Error - unknown MoveType=',
     &           MoveType,' returned from PERTURB.'
              goto 20
            endif
            call chainbuild
          else
            MoveType=0
            moves(0)=moves(0)+1
            nstart_grow=iran_num(3,nres)
            if (print_mc.gt.0)
     &        write (iout,'(2a,i3)') 'Random-generated conformation',
     &        ' - chain regrown from residue',nstart_grow
            call gen_rand_conf(nstart_grow,*30)
          endif
          call geom_to_var(nvar,varia)
cd        write (iout,'(a)') 'New variables:'
cd        write (iout,'(10f8.1)') (rad2deg*varia(i),i=1,nvar)
          ngen=ngen+1
#ifdef MPL
          if (fgProcs.gt.1) then
C Request energy computation from slave processors.
            varia(nvar+1)=nf
            varia(nvar+2)=-1
            varia(nvar+3)=icg
            call mp_bcast(varia(1),msglen_var,BossID,fgGroupID)
          endif
#endif
          call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &         edihcnstr,ecorr,etot)
c         call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
c    &                     edihcnstr,ecorr,etot)
c         write (iout,'(2(a,1pe14.5))') 'Etot=',Etot,' Elowest=',Elowest
          if (etot-elowest.gt.overlap_cut) then
            write (iout,'(a,1pe14.5)') 
     &      'Overlap detected in the current conf.; energy is',etot
            neneval=neneval+1 
            accepted=.false.
            noverlap=noverlap+1
            if (noverlap.gt.maxoverlap) then
              write (iout,'(a)') 'Too many overlapping confs.'
              goto 20
            endif
          else
            if (minim) then
              call minimize(etot,varia,iretcode,nfun)
cd            write (iout,*) 'etot from MINIMIZE:',etot
cd            write (iout,'(a)') 'Variables after minimization:'
cd            write (iout,'(10f8.1)') (rad2deg*varia(i),i=1,nvar)
#ifdef MPL
             if (fgProcs.gt.1) then
C Request energy computation from slave processors.
               varia(nvar+1)=nf
               varia(nvar+2)=-1
               varia(nvar+3)=icg
               call mp_bcast(varia(1),msglen_var,BossID,fgGroupID)
             endif
#endif
              call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                    edihcnstr,ecorr,etot)
              neneval=neneval+nfun+2
            endif
c           call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
c    &                     edihcnstr,ecorr,etot)
            write (iout,'(a,i6,a,1pe16.6)') 'Conformation:',ngen,
     &      ' energy:',etot
C--------------------------------------------------------------------------
C... Do Metropolis test
C--------------------------------------------------------------------------
            accepted=.false.
            my_conf=.false.
#ifdef MPL
            if (Kwita.eq.0) call recv_stop_sig(Kwita)

#endif
            if (WhatsUp.eq.0 .and. Kwita.eq.0) then
              call metropolis(nvar,varia,varold,etot,eold,accepted,
     &                      my_conf,EneLower,it)
            endif
            write (iout,*) 'My_Conf=',My_Conf,' EneLower=',EneLower
            if (accepted) then
#ifdef MPL
!              if (MyID.eq.MasterID) then
                if (My_Conf .and. EneLower) then
                  write (iout,*) 'Calling add2cache from MCM:1.'
                  call add2cache(maxvar,max_cache,ncache,nvar,nphi,
     &           MyID,CachSrc,isent,iused,1,etot,varia,
     &            cache_update,ecache,xcache)
                 write (iout,*) 'ncache=',ncache
                  cache_update=.true.
                endif
                if (cache_update) then
!6/19                  if (Kwita.eq.0 .and. WhatsUp.eq.0) call send_cache
                  cache_update=.false.
                endif
!              endif
#endif
              nacc=nacc+1
              nacc_tot=nacc_tot+1
              if (elowest.gt.etot) elowest=etot
              moves_acc(MoveType)=moves_acc(MoveType)+1
              if (MoveType.eq.1) then
                nbond_acc(nbond)=nbond_acc(nbond)+1
              endif
C Check against conformation repetitions.
              irepet=conf_comp(varia,etot)
c             call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
c    &                       edihcnstr,ecorr,etot)
              if (print_stat) then
#ifdef AIX
              open (istat,file=statname,position='append')
#else
              open (istat,file=statname,access='append')
#endif
              endif
              call statprint(nacc,nfun,iretcode,etot,elowest)
              if (refstr) then
                call var_to_geom(nvar,varia)
                call chainbuild
                call fitsq(rms,c(1,nstart_seq),cref(1,nstart_sup),
     &                    nsup,przes,obrot,non_conv)
                rms=dsqrt(rms)
                call contact(.false.,ncont,icont)
                frac=contact_fract(ncont,ncont_ref,icont,icont_ref)
                write (iout,'(a,f8.3,a,f8.3)') 
     &          'RMS deviation from the reference structure:',rms,
     &          ' % of native contacts:',frac*100
              endif ! refstr
              if (My_Conf) then
                nout=nout+1
                write (iout,*) 'Writing new conformation',nout
                if (refstr) then
                  write (istat,'(i5,10(1pe14.5))') nout,evdw,evdw2,
     &             evdw1+ees,ecorr,ebe,escloc,etors,etot,rms,frac
                else
                  if (print_stat)
     &            write (istat,'(i5,9(1pe14.5))') nout,
     &             evdw,evdw2,evdw1+ees,ecorr,ebe,escloc,etors,etot
                endif ! refstr
                if (print_stat) close(istat)
C Print internal coordinates.
                if (print_int) call briefout(nout,etot)
C Accumulate the newly accepted conf in the coord1 array, if it is different
C from all confs that are already there.
                call compare_s1(n_thr,max_thread2,etot,varia,ii,
     &           enetb1,coord1,rms_deform,.true.,.true.,iprint)
                write (iout,*) 'After compare_ss: n_thr=',n_thr
                if (ii.eq.1 .or. ii.eq.3) then
                  write (iout,'(8f10.4)') 
     &                (rad2deg*coord1(i,n_thr),i=1,nvar)
                endif
              else
                write (iout,*) 'Conformation from cache, not written.'
              endif ! My_Conf 
#ifdef MPL
              call recv_stop_sig(Kwita)
              if (MyID.ne.MasterID .and. Kwita.eq.0) then
cd              print *,'Processor:',MyID,' STOP=',Kwita
                if (irepet.eq.0) then
                  call send_MCM_info(2)
                else
                  call send_MCM_info(1)
                endif
!                if (My_Conf .and. EneLower) 
!     &              call send_conf(nvar,etot,varia,MasterID)
              endif
#endif
              if (nrepm.gt.maxrepm) then
                write (iout,'(a)') 'Too many conformation repetitions.'
                goto 20
              endif
C Store the accepted conf. and its energy.
              eold=etot
              do i=1,nvar
                varold(i)=varia(i)
              enddo
              if (irepet.eq.0) call zapis(varia,etot)
C Lower the temperature, if necessary.
              call cool
#ifdef MPL
!              ncache=0
#endif
            else
#ifdef MPL
              if (MyID.eq.MasterID .and. cache_update) then
!/6/19                if (Kwita.eq.0 .and. WhatsUp.eq.0) call send_cache
                cache_update=.false.
              endif
#endif
              ntrial=ntrial+1
            endif ! accepted
          endif ! overlap
#ifdef MPL
          if (Kwita.eq.0) call recv_stop_sig(kwita)
          if (MyID.eq.MasterID) then
            call receive_MCM_info
            call receive_conf
            if (nacc_tot.ge.maxacc) accepted=.true.
          endif
          if (WhatsUp.ne.0 .or. Kwita.lt.0) accepted=.true.
#endif
   30     continue
        enddo ! accepted
C Check for time limit.
        if (ovrtim()) WhatsUp=-1
        not_done = (nacc_tot.lt.maxacc) .and. (WhatsUp.eq.0)
     &       .and. (Kwita.eq.0)
#ifdef MPL    
        if (Kwita.lt.0) then
          print *,'Processor',MyID,
     &    ' has received STOP signal =',Kwita,' in MCM.'
cd        print *,'not_done=',not_done
          if (Kwita.lt.-1) WhatsUp=Kwita
        else if (nacc_tot.ge.maxacc) then
          print *,'Processor',MyID,' calls send_stop_sig,',
     &     ' because a sufficient # of confs. have been collected.'
cd        print *,'not_done=',not_done
          call send_stop_sig(-1)
        else if (WhatsUp.eq.-1) then
          print *,'Processor',MyID,
     &               ' calls send_stop_sig because of timeout.'
cd        print *,'not_done=',not_done
          call send_stop_sig(-2)
        endif
        if (MyID.eq.MasterID) then
! Send the confs to processors than haven't produced it; one conf per each
! processor only!
          do i=2,nctasks
            ifeed(i)=0
          enddo
          do j=1,ncache
            if (isent(j).eq.0) then
              do i=2,nctasks
!                if (ifeed(i).eq.0) then
                  iproc=cglist(i)
                  if (iproc.ne.CachSrc(j)) then
                    write (iout,*) 'j=',j,' i=',i,' iproc=',iproc
                    call send_conf(nvar,ecache(j),xcache(1,j),iproc)
                    ifeed(i)=ifeed(i)+1
                    isent(j)=isent(j)+1
                    iused(j)=iused(j)+1
                  endif
!                endif
              enddo
            endif
          enddo
        else
          do j=1,ncache
            if (isent(j).eq.0 .and. CachSrc(j).ne.MasterID) then
              isent(j)=isent(j)+1
              call send_conf(nvar,ecache(j),xcache(1,j),MasterID) 
            endif
          enddo
        endif
          
#endif
      enddo ! not_done
      goto 21
   20 WhatsUp=-3
#ifdef MPL
      if (Kwita.eq.0) then
        write (iout,*) 'Processor',MyID,
     &       ' is broadcasting ERROR-STOP signal.'
        write (*,*) 'Processor',MyID,
     &       ' is broadcasting ERROR-STOP signal.'
        call send_stop_sig(-3)
      endif
#endif
   21 continue
      runtime=tcpu()
      write (iout,'(/80(1h*)/20x,a)') 'Summary run statistics:'
      call statprint(nacc,nfun,iretcode,etot,elowest)
      write (iout,'(a)') 
     & 'Statistics of multiple-bond motions. Total motions:' 
      write (iout,'(16i5)') (nbond_move(i),i=1,Nbm)
      write (iout,'(a)') 'Accepted motions:'
      write (iout,'(16i5)') (nbond_acc(i),i=1,Nbm)
      if (it.ge.maxacc)
     &write (iout,'(/80(1h*)/20x,a/80(1h*)/)') 'All iterations done.'
#ifdef MPL
C Safety check whether any information is left. This could cause deadlock.
      if (MyID.eq.MasterID) then
        not_done=.true.
        do while (not_done)
          Kwita=0
          nacc_tot=0
          ncache=0
          call recv_stop_sig(Kwita)
          call receive_mcm_info
          call receive_conf
          not_done=(Kwita+nacc_tot+ncache.ne.0)
        enddo
      endif ! MyID.eq.MasterID
#endif
      return
      end
c------------------------------------------------------------------------------
      subroutine statprint(it,nfun,iretcode,etot,elowest)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.CONTROL'
      include 'COMMON.MCM'
      if (minim) then
        write (iout,'(80(1h*)/a,i5,a,1pe14.5,a,1pe14.5/a,i3,a,i10,
     &  a,i5,a,i5)')
     &  'Finished iteration #',it,' energy is',etot,
     &  ' lowest energy:',elowest,
     &  'SUMSL return code:',iretcode,
     &  ' # of energy evaluations:',neneval,
     &  '# of temperature jumps:',ntherm,
     &  ' # of minima repetitions:',nrepm
      else
        write (iout,'(80(1h*)/a,i8,a,1pe14.5,a,1pe14.5)')
     &  'Finished iteration #',it,' energy is',etot,
     &  ' lowest energy:',elowest
      endif
      write (iout,'(/4a)')
     & 'Kind of move   ','           total','       accepted',
     & '  fraction'
      write (iout,'(58(1h-))')
      do i=-1,MaxMoveType+1
        write(iout,'(a,2i15,f10.5)')MovTypID(i),moves(i),moves_acc(i),
     &         dfloat(moves_acc(i))/dfloat(moves(i))
      enddo
      write (iout,'(a,2i15,f10.5)') 'total           ',nmove,nacc_tot,
     &         dfloat(nacc_tot)/dfloat(nmove)
      write (iout,'(58(1h-))')
      write (iout,'(a,1pe12.4)') 'Elapsed time:',tcpu()
      return
      end
c------------------------------------------------------------------------------
      subroutine heat(over)
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.IOUNITS'
      logical over
C Check if there's a need to increase temperature.
      if (ntrial.gt.maxtrial) then
        if (NstepH.gt.0) then
          if (dabs(Tcur-TMax).lt.1.0D-7) then
            write (iout,'(/80(1h*)/a,f8.3,a/80(1h*))') 
     &      'Upper limit of temperature reached. Terminating.'
            over=.true.
          else
            Tcur=Tcur*TstepH
            if (Tcur.gt.Tmax) Tcur=Tmax
            betbol=1.0D0/(Rbol*Tcur)
            write (iout,'(/80(1h*)/a,f8.3,a,f10.5/80(1h*))')
     &      'System heated up to ',Tcur,' K; BetBol:',betbol
            ntherm=ntherm+1
            ntrial=0
            over=.false.
          endif
        else
         write (iout,'(a)') 
     & 'Maximum number of trials in a single MCM iteration exceeded.'
         over=.true.
        endif
      else
        over=.false.
      endif
      return
      end
c------------------------------------------------------------------------------
      subroutine cool
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.IOUNITS'
      if (nstepC.gt.0 .and. dabs(Tcur-Tmin).gt.1.0D-7) then
        Tcur=Tcur/TstepC
        if (Tcur.lt.Tmin) Tcur=Tmin
        betbol=1.0D0/(Rbol*Tcur)
        write (iout,'(/80(1h*)/a,f8.3,a,f10.5/80(1h*))')
     &  'System cooled down up to ',Tcur,' K; BetBol:',betbol
      endif
      return
      end
C---------------------------------------------------------------------------
      subroutine zapis(varia,etot)
      include 'DIMENSIONS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.MCM'
      include 'COMMON.INFO'
      include 'COMMON.IOUNITS'
      include 'COMMON.CACHE'
      double precision varia(maxvar)
      integer itemp1(MaxSave),itemp2(MaxSave)
      logical lprint
      do i=1,MaxSave
        itemp1(i)=0
        itemp2(i)=0
      enddo
      lprint=.false.
      if (lprint) then
      write (iout,'(a,i5,a,i5)') 'Enter ZAPIS NSave=',Nsave,
     &  ' MaxSave=',MaxSave
      write (iout,'(a)') 'Current energy and conformation:'
      write (iout,'(1pe14.5)') etot
      write (iout,'(10f8.3)') (rad2deg*varia(i),i=1,nvar)
      endif
C Shift the contents of the esave and varsave arrays if filled up.
      write (iout,*) 'Calling add2cache - adding to VARSAVE.'
      call add2cache(maxvar,maxsave,nsave,nvar,nphi,MyID,origin,
     &       itemp1,itemp2,0,etot,varia,cache_update,esave,varsave)
      if (lprint) then
      write (iout,'(a)') 'Energies and the VarSave array.'
      do i=1,nsave
        write (iout,'(i5,1pe14.5)') i,esave(i)
        write (iout,'(10f8.3)') (rad2deg*varsave(j,i),j=1,nvar)
      enddo
      endif
      return
      end 
C---------------------------------------------------------------------------
      subroutine perturb(error,lprint,MoveType,nbond,max_phi)
      include 'DIMENSIONS'
      parameter (MMaxSideMove=100)
      include 'COMMON.MCM'
      include 'COMMON.CHAIN'
      include 'COMMON.INTERACT'
      include 'COMMON.VAR'
      include 'COMMON.GEO'
      include 'COMMON.NAMES'
      include 'COMMON.IOUNITS'
      logical error,lprint,fail
      integer MoveType,nbond,end_select,ind_side(MMaxSideMove)
      double precision max_phi
      double precision psi,gen_psi
      external iran_num
      integer iran_num
      integer ifour /4/
      error=.false.
      lprint=.false.
C Perturb the conformation according to a randomly selected move.
      call SelectMove(MoveType)
cd    write (iout,*) 'MoveType=',MoveType
      itrial=0
      goto (100,200,300,400,500) MoveType
C------------------------------------------------------------------------------
C Backbone N-bond move.
C Select the number of bonds (length of the segment to perturb).
  100 continue
      if (itrial.gt.1000) then
        write (iout,'(a)') 'Too many attempts at multiple-bond move.'
        error=.true.
        return
      endif
      bond_prob=ran_number(0.0D0,sumpro_bond(nbm))
c     print *,'sumpro_bond(nbm)=',sumpro_bond(nbm),
c    & ' Bond_prob=',Bond_Prob
      do i=1,nbm-1
c       print *,i,Bond_Prob,sumpro_bond(i),sumpro_bond(i+1)
        if (bond_prob.ge.sumpro_bond(i) .and. 
     &               bond_prob.le.sumpro_bond(i+1)) then
          nbond=i+1
          goto 10
        endif
      enddo
      write (iout,'(2a)') 'In PERTURB: Error - number of bonds',
     &                    ' to move out of range.'
      error=.true.
      return
   10 continue
      if (nwindow.gt.0) then
C Select the first residue to perturb
        iwindow=iran_num(1,nwindow)
        print *,'iwindow=',iwindow
        iiwin=1
        do while (winlen(iwindow).lt.nbond)
          iwindow=iran_num(1,nwindow)
          iiwin=iiwin+1
          if (iiwin.gt.1000) then
             write (iout,'(a)') 'Cannot select moveable residues.'
             error=.true.
             return
          endif
        enddo 
        nstart=iran_num(winstart(iwindow),winend(iwindow))
      else
        nstart = iran_num(koniecl+2,nres-nbond-koniecl)  
cd      print *,'nres=',nres,' nbond=',nbond,' koniecl=',koniecl,
cd   &        ' nstart=',nstart
      endif
      psi = gen_psi()
      if (psi.eq.0.0) then
        error=.true.
        return
      endif
      if (print_mc.gt.1) write (iout,'(a,i4,a,i4,a,f8.3)')
     & 'PERTURB: nbond=',nbond,' nstart=',nstart,' psi=',psi*rad2deg
cd    print *,'nstart=',nstart
      call bond_move(nbond,nstart,psi,.false.,error)
      if (error) then 
        write (iout,'(2a)') 
     & 'Could not define reference system in bond_move, ',
     & 'choosing ahother segment.'
        itrial=itrial+1
        goto 100
      endif
      nbond_move(nbond)=nbond_move(nbond)+1
      moves(1)=moves(1)+1
      nmove=nmove+1
      return
C------------------------------------------------------------------------------
C Backbone endmove. Perturb a SINGLE angle of a residue close to the end of
C the chain.
  200 continue
      lprint=.true.
c     end_select=iran_num(1,2*koniecl)
c     if (end_select.gt.koniecl) then
c       end_select=nphi-(end_select-koniecl)
c     else 
c       end_select=koniecl+3
c     endif
c     if (nwindow.gt.0) then
c       iwin=iran_num(1,nwindow)
c       i1=max0(4,winstart(iwin))
c       i2=min0(winend(imin)+2,nres)
c       end_select=iran_num(i1,i2)
c     else
        end_select=iran_num(4,nres)
c     endif
      psi=max_phi*gen_psi()
      if (psi.eq.0.0D0) then
        error=.true.
        return
      endif
      phi(end_select)=pinorm(phi(end_select)+psi)
      if (print_mc.gt.1) write (iout,'(a,i4,a,f8.3,a,f8.3)') 
     & 'End angle',end_select,' moved by ',psi*rad2deg,' new angle:',
     & phi(end_select)*rad2deg   
c     if (end_select.gt.3) 
c    &   theta(end_select-1)=gen_theta(itype(end_select-2),
c    &                              phi(end_select-1),phi(end_select))
c     if (end_select.lt.nres) 
c    &    theta(end_select)=gen_theta(itype(end_select-1),
c    &                              phi(end_select),phi(end_select+1))
cd    print *,'nres=',nres,' end_select=',end_select
cd    print *,'theta',end_select-1,theta(end_select-1)
cd    print *,'theta',end_select,theta(end_select)
      moves(2)=moves(2)+1
      nmove=nmove+1
      lprint=.false.
      return
C------------------------------------------------------------------------------
C Side chain move.
C Select the number of SCs to perturb.
  300 isctry=0 
  301 nside_move=iran_num(1,MaxSideMove) 
c     print *,'nside_move=',nside_move,' MaxSideMove',MaxSideMove
C Select the indices.
      do i=1,nside_move
        icount=0
  111   inds=iran_num(nnt,nct) 
        icount=icount+1
        if (icount.gt.1000) then
          write (iout,'(a)')'Error - cannot select side chains to move.'
          error=.true.
          return
        endif
        if (itype(inds).eq.10) goto 111
        do j=1,i-1
          if (inds.eq.ind_side(j)) goto 111
        enddo
        do j=1,i-1
          if (inds.lt.ind_side(j)) then
            indx=j
            goto 112
          endif
        enddo
        indx=i
  112   do j=i,indx+1,-1
          ind_side(j)=ind_side(j-1)
        enddo 
  113   ind_side(indx)=inds
      enddo
C Carry out perturbation.
      do i=1,nside_move
        ii=ind_side(i)
        iti=itype(ii)
        call gen_side(iti,theta(ii+1),alph(ii),omeg(ii),fail)
        if (fail) then
          isctry=isctry+1
          if (isctry.gt.1000) then
            write (iout,'(a)') 'Too many errors in SC generation.'
            error=.true.
            return
          endif
          goto 301 
        endif
        if (print_mc.gt.1) write (iout,'(2a,i4,a,2f8.3)') 
     &   'Side chain ',restyp(iti),ii,' moved to ',
     &   alph(ii)*rad2deg,omeg(ii)*rad2deg
      enddo
      moves(3)=moves(3)+1
      nmove=nmove+1
      return
C------------------------------------------------------------------------------
C THETA move
  400 end_select=iran_num(3,nres)
      theta_new=gen_theta(itype(end_select),phi(end_select),
     &                    phi(end_select+1))
      if (print_mc.gt.1) write (iout,'(a,i3,a,f8.3,a,f8.3)') 
     & 'Theta ',end_select,' moved from',theta(end_select)*rad2deg,
     & ' to ',theta_new*rad2deg
      theta(end_select)=theta_new  
      moves(4)=moves(4)+1
      nmove=nmove+1 
      return
C------------------------------------------------------------------------------
C Error returned from SelectMove.
  500 error=.true.
      return
      end
C------------------------------------------------------------------------------
      subroutine SelectMove(MoveType)
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.IOUNITS'
      what_move=ran_number(0.0D0,sumpro_type(MaxMoveType))
      do i=1,MaxMoveType
        if (what_move.ge.sumpro_type(i-1).and.
     &            what_move.lt.sumpro_type(i)) then
          MoveType=i
          return
        endif
      enddo
      write (iout,'(a)') 
     & 'Fatal error in SelectMoveType: cannot select move.'
      MoveType=MaxMoveType+1
      return
      end
c----------------------------------------------------------------------------
      double precision function gen_psi()
      implicit none
      integer i
      double precision x,ran_number
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      x=0.0D0
      do i=1,100
        x=ran_number(-pi,pi)
        if (dabs(x).gt.angmin) then
          gen_psi=x
          return
        endif
      enddo
      write (iout,'(a)')'From Gen_Psi: Cannot generate angle increment.'
      gen_psi=0.0D0
      return
      end
c----------------------------------------------------------------------------
      subroutine metropolis(n,xcur,xold,ecur,eold,accepted,my_conf,
     &                      enelower,iter)
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.IOUNITS'
      include 'COMMON.VAR'
      include 'COMMON.GEO'
#ifdef MPL
      include 'COMMON.INFO'
      include 'COMMON.CACHE'
#endif
      double precision ecur,eold,xx,ran_number,bol
      double precision xcur(n),xold(n)
      double precision ecut1 /-1.0D-4/,ecut2 /5.0D-3/,tola/5.0D0/
      logical accepted,my_conf,not_done,enelower
      logical lprn
C Set lprn=.true. for debugging.
      lprn=.false.
#ifdef MPL
C See whether other processors haven't come with a lower energy.

      elow=dmin1(ecur,eold)

!      if (MyID.ne.MasterID) call receive_cache
      if (MyID.ne.MasterID) call receive_conf

      not_done=(ncache.gt.0 .and. mod(iter,50).eq.0)

      my_conf=.true.

      do while (not_done)

      NotMe=1
      do while (NotMe.le.ncache .and. MyID.eq.CachSrc(NotMe) 
     &   .and. iused(NotMe).gt.0)
        NotMe=NotMe+1
      enddo ! NotMe
cd    write (iout,*) 'Enter Metropolis, ncache=',ncache,' NotMe=',NotMe
cd    write (iout,*) 'The CachSrc array:'
cd    write (iout,*) (CachSrc(i),i=1,ncache)
cd    write (iout,*) 'Ecur=',ecur,' elow=',elow,' ecache=',ecache(NotMe)
      if (NotMe.le.ncache .and. ecache(NotMe).lt.elow) 
     &   then
!        difene=ecache(NotMe)-eold
!        reldife=difene/dmax1(dabs(eold),dabs(ecache(NotMe)),1.0D0)
!        difa=dif_ang(nphi,xcache(1,NotMe),xold)
cd      write (iout,*) 'eold',eold,' ecache',ecache(NotMe),' difa',difa
cd      write (iout,*) 'xold array'
cd      write (iout,'(10f8.3)') (rad2deg*xold(i),i=1,nphi)
cd      write (iout,*) 'xcache array'
cd      write (iout,'(10f8.3)') (rad2deg*xcache(i,NotMe),i=1,nphi)
!        if (reldife.gt.ecut1 .and. difa.lt.tola) then
!          write (iout,*) 'Cache conformation',NotMe,
!     &   ' is too similar to last accepted conf and will be removed.'
!          write (iout,*) 'Energies are',eold,ecache(NotMe),
!     &     ' diff. in angles',difa 
!          call rm_from_cache(NotMe,maxvar,max_cache,ncache,nvar,CachSrc,
!     &                 ecache,xcache)
!          if (ncache.eq.0) then
!            not_done=.false.
!!            my_conf=.true.
!          endif
!        else
! 6/19/98
! Conformation is going to be used; we note this so as not to use it next
! time
          iused(NotMe)=iused(NotMe)+1
          ecur=ecache(NotMe)
          do i=1,n
            xcur(i)=xcache(i,NotMe)
          enddo
          write (iout,*) 
     &        'Current conformation replaced by cache, ecur=',ecur
          write (iout,'(10f8.3)') (rad2deg*xcur(i),i=1,n)
          my_conf=.false. 
          not_done=.false.
!        endif 
      else
        not_done=.false.
        my_conf=.true.
      endif

      enddo ! not_done
#else
      my_conf=.true.
#endif               
C Check if the conformation is similar.
      difene=ecur-eold
      reldife=difene/dmax1(dabs(eold),dabs(ecur),1.0D0)
      if (lprn) then
        write (iout,*) 'Metropolis'
        write(iout,*)'ecur,eold,difene,reldife',ecur,eold,difene,reldife
      endif
C If energy went down remarkably, we accept the new conformation 
C unconditionally.
      if (reldife.lt.ecut1) then
        accepted=.true.
        EneLower=.true.
        write (iout,'(a)') 
     &   'Conformation accepted, because energy has lowered remarkably.'
      elseif (reldife.lt.ecut2 .and. dif_ang(nphi,xcur,xold).lt.tola) 
     & then
C Reject the conf. if energy has changed insignificantly and there is not 
C much change in conformation.
        write (iout,'(2a)') 'Conformation rejected, because it is',
     &      ' similar to the preceding one.'
        accepted=.false.
      else 
C Else carry out Metropolis test.
        EneLower=.false.
        xx=ran_number(0.0D0,1.0D0) 
        xxh=betbol*difene
        if (lprn)
     &    write (iout,*) 'betbol=',betbol,' difene=',difene,' xxh=',xxh
        if (xxh.gt.50.0D0) then
          bol=0.0D0
        else
          bol=exp(-xxh)
        endif
        if (lprn) write (iout,*) 'bol=',bol,' xx=',xx
        if (bol.gt.xx) then
          accepted=.true. 
          write (iout,'(a)') 
     &    'Conformation accepted, because it passed Metropolis test.'
        else
          accepted=.false.
          write (iout,'(a)') 
     & 'Conformation rejected, because it did not pass Metropolis test.'
        endif
      endif
      return
      end 
c------------------------------------------------------------------------------
      integer function conf_comp(x,ene)
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO' 
      double precision etol /0.1D0/, angtol /20.0D0/
      double precision x(maxvar)
      double precision dif_ang,difa
      do ii=nsave,1,-1
c       write (iout,*) 'ii=',ii,'ene=',ene,esave(ii),dabs(ene-esave(ii))
        if (dabs(ene-esave(ii)).lt.etol) then
          difa=dif_ang(nphi,x,varsave(1,ii))
c         do i=1,nphi
c           write(iout,'(i3,3f8.3)')i,rad2deg*x(i),
c    &          rad2deg*varsave(i,ii)
c         enddo
c         write(iout,*) 'ii=',ii,' difa=',difa,' angtol=',angtol
          if (difa.le.angtol) then
            if (print_mc.gt.0) then
            write (iout,'(a,i5,2(a,1pe15.4))') 
     &      'Current conformation matches #',ii,
     &      ' in the store array ene=',ene,' esave=',esave(ii)
c           write (*,'(a,i5,a)') 'Current conformation matches #',ii,
c    &      ' in the store array.'
            endif ! print_mc.gt.0
            if (print_mc.gt.1) then
            do i=1,nphi
              write(iout,'(i3,3f8.3)')i,rad2deg*x(i),
     &            rad2deg*varsave(i,ii)
            enddo
            endif ! print_mc.gt.1
            nrepm=nrepm+1
            conf_comp=ii
            return
          endif
        endif
      enddo 
      conf_comp=0
      return
      end 
C----------------------------------------------------------------------------
      double precision function dif_ang(n,x,y)
      implicit none
      integer i,n
      double precision x(n),y(n)
      double precision w,wa,dif,difa
      double precision pinorm 
      include 'COMMON.GEO'
      wa=0.0D0
      difa=0.0D0
      do i=1,n
        dif=dabs(pinorm(y(i)-x(i)))
        if (dabs(dif-dwapi).lt.dif) dif=dabs(dif-dwapi)
        w=1.0D0-(2.0D0*(i-1)/(n-1)-1.0D0)**2+1.0D0/n
        wa=wa+w
        difa=difa+dif*dif*w
      enddo 
      dif_ang=rad2deg*dsqrt(difa/wa)
      return
      end
c--------------------------------------------------------------------------
      subroutine add2cache(n1,n2,ncache,nvar,nphi,SourceID,CachSrc,
     &             isent,iused,iu,ecur,xcur,cache_update,ecache,xcache)
      include 'DIMENSIONS'
      include 'COMMON.INFO'
      include 'COMMON.GEO'
      include 'COMMON.IOUNITS'
      integer n1,n2,ncache,nvar,nphi,SourceID,CachSrc(n2),isent(n2),
     &  iused(n2),iu
      logical cache_update
      integer i,ii,j
      double precision ecur,xcur(nvar),ecache(n2),xcache(n1,n2) 
      double precision difa,dif_ang
      double precision etol /0.1D0/, angtol /20.0D0/
      logical lprn 

      lprn=.false.
      write (iout,*) 'Adding conformation to cache, energy',ecur
      if (lprn) then
      write (iout,*) 'Enter ADD2CACHE ncache=',ncache ,' ecur',ecur
      write (iout,'(10f8.3)') (rad2deg*xcur(i),i=1,nvar)
      write (iout,*) 'Old CACHE array:'
      do i=1,ncache
        write(iout,*) 'i=',i,' ecache=',ecache(i),' CachSrc',CachSrc(i),
     &   ' isent',isent(i),' iused',iused(i)
        write (iout,'(10f8.3)') (rad2deg*xcache(j,i),j=1,nvar)
      enddo
      endif

! 6/19/98 Check if the current conformation is not similar to any in cache

      cache_update=.false.

      do i=1,ncache

        if (dabs(ecur - ecache(i)).lt.etol) then
       
           difa=dif_ang(nphi,xcache(1,i),xcur)
           if (difa.lt.angtol) then
            write (iout,*) 'Conformation found similar to conformation',
     &        i,' that is already in cache; cache not updated.'
             return
           endif

        endif

      enddo


! New conformation is different; cache will be updated

      cache_update=.true.

      i=ncache
      do while (i.gt.0 .and. ecur.lt.ecache(i))
        i=i-1
      enddo
      i=i+1
cd    write (iout,*) 'i=',i,' ncache=',ncache
      if (ncache.eq.n2) then
        write (iout,*) 'Cache dimension exceeded',ncache,n2
        write (iout,*) 'Highest-energy conformation will be removed.'
        ncache=ncache-1
      endif
      do ii=ncache,i,-1
        ecache(ii+1)=ecache(ii)
        CachSrc(ii+1)=CachSrc(ii)
        isent(ii+1)=isent(ii)
        iused(ii+1)=iused(ii)
        do j=1,nvar
          xcache(j,ii+1)=xcache(j,ii)
        enddo
      enddo
      ecache(i)=ecur
      CachSrc(i)=SourceID
      isent(i)=0
      iused(i)=iu
      do j=1,nvar
        xcache(j,i)=xcur(j)
      enddo
      ncache=ncache+1
      if (lprn) then
      write (iout,*) 'New CACHE array:'
      do i=1,ncache
        write(iout,*) 'i=',i,' ecache=',ecache(i),' CachSrc',CachSrc(i),
     &   ' isent',isent(i),' iused',iused(i)
        write (iout,'(10f8.3)') (rad2deg*xcache(j,i),j=1,nvar)
      enddo
      endif
      return
      end
c--------------------------------------------------------------------------
      subroutine rm_from_cache(i,n1,n2,ncache,nvar,CachSrc,ecache,
     &                         xcache)
      implicit none 
      include 'COMMON.GEO'
      include 'COMMON.IOUNITS'
      integer n1,n2,ncache,nvar,CachSrc(n2)
      integer i,ii,j
      double precision ecache(n2),xcache(n1,n2) 

cd    write (iout,*) 'Enter RM_FROM_CACHE'
cd    write (iout,*) 'Old CACHE array:'
cd    do ii=1,ncache
cd    write (iout,*)'i=',ii,' ecache=',ecache(ii),' CachSrc',CachSrc(ii)
cd      write (iout,'(10f8.3)') (rad2deg*xcache(j,ii),j=1,nvar)
cd    enddo

      do ii=i+1,ncache
        ecache(ii-1)=ecache(ii)
        CachSrc(ii-1)=CachSrc(ii)
        do j=1,nvar
          xcache(j,ii-1)=xcache(j,ii)
        enddo
      enddo
      ncache=ncache-1
cd    write (iout,*) 'New CACHE array:'
cd    do i=1,ncache
cd      write (iout,*) 'i=',i,' ecache=',ecache(i),' CachSrc',CachSrc(i)
cd      write (iout,'(10f8.3)') (rad2deg*xcache(j,i),j=1,nvar)
cd    enddo
      return
      end
