      subroutine minimize(etot,x,iretcode,nfun)
      include 'DIMENSIONS'
      parameter (liv=60,lv=(77+maxvar*(maxvar+17)/2)) 
*********************************************************************
* OPTIMIZE sets up SUMSL or DFP and provides a simple interface for *
* the calling subprogram.                                           *     
* when d(i)=1.0, then v(35) is the length of the initial step,      *     
* calculated in the usual pythagorean way.                          *     
* absolute convergence occurs when the function is within v(31) of  *     
* zero. unless you know the minimum value in advance, abs convg     *     
* is probably not useful.                                           *     
* relative convergence is when the model predicts that the function *   
* will decrease by less than v(32)*abs(fun).                        *   
*********************************************************************
      include 'COMMON.IOUNITS'
      include 'COMMON.VAR'
      include 'COMMON.GEO'
      include 'COMMON.MINIM'
      common /srutu/ icall
      dimension iv(liv)                                               
      double precision minval,x(maxvar),d(maxvar),v(1:lv)                     
      external func,gradient,fdum
      logical not_done,change,reduce 

      icall = 1

      NOT_DONE=.TRUE.

c     DO WHILE (NOT_DONE)

      call deflt(2,iv,liv,lv,v)                                         
* 12 means fresh start, dont call deflt                                 
      iv(1)=12                                                          
* max num of fun calls                                                  
      if (maxfun.eq.0) maxfun=500
      iv(17)=maxfun
* max num of iterations                                                 
      if (maxmin.eq.0) maxmin=1000
      iv(18)=maxmin
* controls output                                                       
      iv(19)=2                                                          
* selects output unit                                                   
c     iv(21)=iout                                                       
      iv(21)=0
* 1 means to print out result                                           
      iv(22)=0                                                          
* 1 means to print out summary stats                                    
      iv(23)=0                                                          
* 1 means to print initial x and d                                      
      iv(24)=0                                                          
* min val for v(radfac) default is 0.1                                  
      v(24)=0.1D0                                                       
* max val for v(radfac) default is 4.0                                  
      v(25)=2.0D0                                                       
c     v(25)=4.0D0                                                       
* check false conv if (act fnctn decrease) .lt. v(26)*(exp decrease)    
* the sumsl default is 0.1                                              
      v(26)=0.1D0
* false conv if (act fnctn decrease) .lt. v(34)                         
* the sumsl default is 100*machep                                       
      v(34)=v(34)/100.0D0                                               
* absolute convergence                                                  
      if (tolf.eq.0.0D0) tolf=1.0D-4
      v(31)=tolf
* relative convergence                                                  
      if (rtolf.eq.0.0D0) rtolf=1.0D-4
      v(32)=rtolf
* controls initial step size                                            
       v(35)=1.0D-1                                                    
* large vals of d correspond to small components of step                
      do i=1,nphi
        d(i)=1.0D-1
      enddo
      do i=nphi+1,nvar
        d(i)=1.0D-1
      enddo
cd    print *,'Calling SUMSL'
c     call var_to_geom(nvar,x)
c     call chainbuild
c     call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,edihcnstr,
c        ecorr,etot)
c     call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
c    &   edihcnstr,ecorr,etot)
      call sumsl(nvar,d,x,func,gradient,iv,liv,lv,v,idum,rdum,fdum)      
      etot=v(10)                                                      
      iretcode=iv(1)
cd    print *,'Exit SUMSL; return code:',iretcode,' energy:',etot
cd    write (iout,'(/a,i4/)') 'SUMSL return code:',iv(1)
c     call intout
c     change=reduce(x)
c     call var_to_geom(nvar,x)
c     if (change) then
c       write (iout,'(a)') 'Reduction worked, minimizing again...'
c     else
c       not_done=.false.
c     endif
c     call chainbuild
c     call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,edihcnstr,
c    &    ecorr,etot)
c     call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,ecorr,
c    &               edihcnstr,etot)
      nfun=iv(6)

c     write (*,*) 'Processor',MyID,' leaves MINIMIZE.'

c     ENDDO ! NOT_DONE

      return  
      end  
#ifdef MPL
c----------------------------------------------------------------------------
      subroutine free_slaves(nfun,varia)
      include 'DIMENSIONS'
      include 'COMMON.INFO'
      include 'COMMON.DERIV'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      double precision varia(maxvar)
      if (fgProcs.gt.1) then
        varia(nvar+1)=-dfloat(nfun)
        varia(nvar+2)=nfl
        varia(nvar+3)=icg
cd      write (*,*) 'Processor',MyID,' sends MP_BCAST in MINIMIZE',
cd   & ' nsglen=',msglen_var,' fgGroupID',fgGroupID
        call mp_bcast(varia(1),msglen_var,MyID,fgGroupID)
cd      write (*,*) 
cd   & 'Processor',MyID,' completed send MP_BCAST in MINIMIZE.'
      endif ! (fgProcs.gt.1)
      return
      end
c------------------------------------------------------------------------------
      subroutine ergastulum
      include 'DIMENSIONS'
      include 'COMMON.INFO'
      include 'COMMON.DERIV'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      double precision x(maxvar),g(maxvar)
      integer idum
      double precision rdum
      external fdum
C Workers wait for variables and NF, and NFL from the boss 
      nf=0
      do while (nf.ge.0)
cd      write (*,*) 'Processor',MyID,' receives MP_BCAST in MINIMIZE',
cd   & ' nsglen=',msglen_var,' fgGroupID',fgGroupID
        call mp_bcast(x(1),msglen_var,BossID,fgGroupID)
cd      write (*,*) 
cd   & 'Processor',MyID,' completed receive MP_BCAST in MINIMIZE.'
        nf=x(nvar+1)
        nfl=x(nvar+2)
        icg=x(nvar+3)
cd      write (iout,*) 'Processor',MyID,' NF',NF,' NFL',NFL,' ICG',ICG
cd      write (*,*) 'Processor',MyID,
cd   &              ' received NF',NF,' NFL',NFL,' ICG',ICG
        if (nf) 50,10,10
   10   if (nfl) 30,40,40
   30   call func(nvar,x,nf,f,idum,rdum,fdum)
        goto 20
   40   call gradient(nvar,x,nf,g,idum,rdum,fdum) 
   20   continue
      enddo ! (nf =< 0)
   50 continue

      write (*,*) 'Processor',MyID,' leves ERGASTULUM.'

      return
      end
#endif
************************************************************************
      subroutine func(n,x,nf,f,uiparm,urparm,ufparm)  
      include 'DIMENSIONS'
      include 'COMMON.DERIV'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      common /chuju/ jjj
      integer jjj
      double precision ufparm
      external ufparm                                                   
      integer uiparm(1)                                                 
      real*8 urparm(1)                                                    
      dimension x(maxvar)
#ifdef MPL
      include 'COMMON.INFO'
cd    write (*,*) 'Processor',MyID,' entered FUNC'
#endif
c     if (jjj.gt.0) then
c       write (iout,'(10f8.3)') (rad2deg*x(i),i=1,n)
c     endif
      nfl=nf
      icg=mod(nf,2)+1
#ifdef MPL
      if (MyID.eq.BossID .and. fgProcs.gt.1) then 
        x(n+1)=dfloat(nf)
        x(n+2)=-1
        x(n+3)=icg
cd      write (*,*) 'Processor',MyID,' sends MP_BCAST in FUNC',
cd   & ' nsglen=',msglen_var,' fgGroupID',fgGroupID
        call mp_bcast(x(1),msglen_var,BossID,fgGroupID)
cd      write (*,*) 'Processor',MyID,' completed send MP_BCAST in FUNC.'
cd      write (*,*) 'Processor',MyID,
cd   &              ' sent NF',NF,' NFL',NFL,' ICG',ICG
      endif
#endif
      call var_to_geom(n,x)
      call zerograd
      call chainbuild
cd    write (iout,*) 'ETOTAL called from FUNC'
      call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,edihcnstr,
     &   ecorr,etot)
      f=etot
c     if (jjj.gt.0) then
c       write (iout,'(10f8.3)') (rad2deg*x(i),i=1,n)
c       write (iout,*) 'f=',etot
c       jjj=0
c     endif
      return                                                            
      end                                                               
