      subroutine read_control
C
C Read molecular data
C
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
      logical OKRandom, prng_restart
      real*8  r1
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.TIME1'
      include 'COMMON.THREAD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.CONTROL'
      include 'COMMON.MCM'
      include 'COMMON.MAP'
      include 'COMMON.CHAIN'
      include 'COMMON.HEADER'
      character*80 ucase
      character*320 controlcard
#ifdef MP
      include 'COMMON.INFO'
      integer SeedType,SeedId,error_msg
C
C Body
C
      SeedType=11
      SeedId=11
#endif

      read (INP,'(a)') titel
      call card_concat(controlcard)

#ifdef MPL 
C Only the master will set random seed.
      if (MyID.eq.MasterId) then
#endif
        call reada(controlcard,'SEED',seed,0.0D0)
        iseed=-dint(dabs(seed))
#ifndef MPL
        write (iout,'(a,i10)') 'Random seed:',iseed
        write (*,'(a,i10)') 'Random seed:',iseed
#endif
        if (iseed.eq.0) then
          write (iout,'(/80(1h*)/20x,a/80(1h*))') 
     &     'Random seed undefined. The program will stop.'
          write (*,'(/80(1h*)/20x,a/80(1h*))') 
     &     'Random seed undefined. The program will stop.'
#ifdef MPL
          call mp_stopall(error_msg)
#elifdef MPI
          call mpi_abort(mpi_comm_world,error_msg,ierr)
#else
          stop 'Bad random seed.'
#endif
        endif
#ifdef MPI
      iseed = iseed*(MyID+1)+1
      write (iout,*) 'MPI: node= ', MyID, ' iseed= ',iseed
      write (*,*) 'MPI: node= ',MyID, ' iseed= ',iseed
      OKRandom = prng_restart(MyID,iseed)
      if (OKRandom) then
        r1 = prng_next(MyID)
        if (r1.lt.0.0) OKRandom=.false.
      endif
      if (.not.OKRandom) then
        write (jout,*) 'PRNG IS NOT WORKING!!!', r1
        write (*,*)    'PRNG IS NOT WORKING!!!', r1
        call mpi_abort(mpi_comm_world,error_msg,ierr)
        stop
      endif
#else
        call vrndst(iseed)
        write (iout,*) 'ran_num',ran_number(0.0d0,1.0d0)
#ifdef MPL
        write (iout,'(a,i10)') 'Random seed:',iseed
        do iproc=fgProcs,nprocs-1,fgProcs
          seed=ran_number(1.0D2,1.0D7)
          iseed=-dabs(seed)
          call mp_send(iseed,4,iproc,SeedType,SeedId) 
        enddo   
      else if (MyID.eq.BossID) then
        call mp_brecv(iseed,4,MasterId,SeedType,SeedId)
        call vrndst(iseed)
        write (iout,'(a,i10)') 'Random seed:',iseed
      endif
#endif
#endif

C Set up the time limit (caution! The time must be input in minutes!)
      call reada(controlcard,'TIMLIM',timlim,60.0D0) 
      write (iout,'(a,f10.1)') 'Time limit (min):',timlim
      timlim=60.0D0*timlim
#ifdef MPL
c      timlim=timlim/nprocs
#endif
        
      if (index(controlcard,'SC_DISTR').gt.0) then
        modecalc=-1
        return
      endif
      modecalc=0
      minim=(index(controlcard,'MINIMIZE').gt.0)
      outpdb=(index(controlcard,'PDBOUT').gt.0)
      print *,'outpdb',outpdb
      outmol2=(index(controlcard,'MOL2OUT').gt.0)
      pdbref=(index(controlcard,'PDBREF').gt.0)
      refstr=pdbref .or. (index(controlcard,'REFSTR').gt.0)
c      call readi(controlcard,'NSTART_SUP',nstart_sup,1)
      indpdb=index(controlcard,'PDBSTART')
      call readi(controlcard,'IPRINT',iprint,0)
      call readi(controlcard,'MAXGEN',maxgen,10000)
      call readi(controlcard,'MAXOVERLAP',maxoverlap,1000)
      if (index(controlcard,'REGULAR').gt.0.0D0) then
        call reada(controlcard,'WEIDIS',weidis,0.1D0)
        modecalc=1
        refstr=.true.
      endif
      if (index(controlcard,'CHECKGRAD').gt.0) then
        modecalc=5
        if (index(controlcard,'CART').gt.0) then
          icheckgrad=1
        elseif (index(controlcard,'CARINT').gt.0) then
          icheckgrad=2
        else
          icheckgrad=3
        endif
      elseif (index(controlcard,'THREAD').gt.0) then
        modecalc=2
        call readi(controlcard,'THREAD',nthread,0)
        if (nthread.gt.0) then
          call reada(controlcard,'WEIDIS',weidis,0.1D0)
        else
          write (iout,'(a)')'A number has to follow the THREAD keyword.'
          stop 'Error termination in Read_Control.'
        endif
      else if (index(controlcard,'MCM').gt.0) then
        modecalc=3
      else if (index(controlcard,'MCE').gt.0) then
        modecalc=6
      else if (index(controlcard,'MULTCONF').gt.0) then
        modecalc=4
      else if (index(controlcard,'MAP').gt.0) then
        modecalc=7
        call readi(controlcard,'MAP',nmap,0)
      endif

      iscode=index(controlcard,'ONE_LETTER')
      indphi=index(controlcard,'PHI')
      indback=index(controlcard,'BACK')
      iranconf=index(controlcard,'RAND_CONF')
cd    write (iout,'(a)') controlcard
cd    write (iout,*) 'IRANCONF',IRANCONF
#ifdef MPL
cd    write (iout,*) 'Processor:',MyId,' ISEED,TIMLIM,MODECALC,ISCODE',
cd   &    iseed,timlim,modecalc,iscode
cd    write (*,*) 'Processor:',MyId,' ISEED,TIMLIM,MODECALC,ISCODE',
cd   &    iseed,timlim,modecalc,iscode
cd    write (iout,*) 'IPDB,INDPHI,INDBACK,IRANCONF,NTHREAD,WEIDIS',
cd   &    indpdb,indphi,indback,iranconf,nthread,weidis
cd    write (*,*) 'IPDB,INDPHI,INDBACK,IRANCONF,NTHREAD,WEIDIS',
cd   &    indpdb,indphi,indback,iranconf,nthread,weidis
#endif
      return
      end
c--------------------------------------------------------------------------
      subroutine molread
C
C Read molecular data.
C
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.CONTACTS'
      include 'COMMON.TIME1'
      include 'COMMON.TORCNSTR'
      include 'COMMON.BOUNDS'
#ifdef MP
      include 'COMMON.INFO'
      integer error_msg
#endif
      character*4 sequence(maxres)
      integer rescode
      double precision x(maxvar)
      character*64 pdbfile
      character*320 weightcard
      character*80 weightcard_t,ucase
      dimension itype_pdb(maxres)
      logical seq_comp
C
C Body
C
C Read weights of the subsequent energy terms.
      call card_concat(weightcard)
      call reada(weightcard,'WLONG',wlong,1.0D0)
      call reada(weightcard,'WSCC',wsc,wlong)
      call reada(weightcard,'WSCP',wscp,wlong)
      call reada(weightcard,'WELEC',welec,1.0D0)
      call reada(weightcard,'WSTRAIN',wstrain,1.0D0)
      call reada(weightcard,'WTOR',wtor,1.0D0)
      call reada(weightcard,'WANG',wang,1.0D0)
      call reada(weightcard,'WSCLOC',wscloc,1.0D0)
      call reada(weightcard,'SCAL14',scal14,0.4D0)
      call reada(weightcard,'SCALSCP',scalscp,1.0d0)
C 12/1/95 Added weight for the multi-body term WCORR
      call reada(weightcard,'WCORR',wcorr,1.0D0)
      write (iout,'(a/5(a,f5.3))') 'Weights of energy terms:',
     &  ' Wsc: ',wsc,' Wscp:',Wscp,' Welec: ',welec,
     &  ' Wstrain: ',wstrain,
     &  ' Wtor: ',wtor,' Wang: ',wang,' Wscloc: ',wscloc,
     &  ' Wcorr: ',wcorr
      write (iout,'(a,f8.3)') 
     &  'Scaling factor of 1,4 SC-p interactions:',scal14
      write (iout,'(a,f8.3)') 
     &  'General scaling factor of SC-p interactions:',scalscp
      do i=1,20
        aad(i,1)=scalscp*aad(i,1)
        aad(i,2)=scalscp*aad(i,2)
        bad(i,1)=scalscp*bad(i,1)
        bad(i,2)=scalscp*bad(i,2)
      enddo
      print *,'indpdb=',indpdb,' pdbref=',pdbref
      if (indpdb.gt.0 .or. pdbref) then
        read(inp,'(a)') pdbfile
        write (iout,'(2a)') 'PDB data will be read from file ',pdbfile
        open(ipdbin,file=pdbfile,status='old',err=33)
        goto 34 
  33    write (iout,'(a)') 'Error opening PDB file.'
        stop
  34    continue
        print *,'Begin reading pdb data'
        call readpdb
        print *,'Finished reading pdb data'
        write (iout,'(a,i3,a,i3)')'nsup=',nsup,' nstart_sup=',nstart_sup
        do i=1,nres
          itype_pdb(i)=itype(i)
        enddo
        close (ipdbin)
      endif
      if (indpdb.eq.0) then
C Read sequence if not taken from the pdb file.
        read (inp,*) nres
        print *,'nres=',nres
        if (iscode.gt.0) then
          read (inp,'(80a1)') (sequence(i)(1:1),i=1,nres)
        else
          read (inp,'(20(1x,a3))') (sequence(i),i=1,nres)
        endif
C Convert sequence to numeric code
        do i=1,nres
          itype(i)=rescode(i,sequence(i),iscode)
        enddo
      endif
      print *,nres
      print '(20i4)',(itype(i),i=1,nres)
      do i=1,nres
	if (itype(i).ne.20) then
	  itel(i)=1
        else
	  itel(i)=2
        endif  
      enddo
      print *,'Call Read_Bridge.'
      call read_bridge
      do i=1,nres
        phibound(1,i)=-pi
        phibound(2,i)=pi
      enddo
      read (inp,*) ndih_constr
      if (ndih_constr.gt.0) then
        read (inp,*) ftors
        read (inp,*) (idih_constr(i),phi0(i),drange(i),i=1,ndih_constr)
        write (iout,*) 
     & 'There are',ndih_constr,' constraints on phi angles.'
        do i=1,ndih_constr
          write (iout,'(i5,2f8.3)') idih_constr(i),phi0(i),drange(i)
        enddo
        do i=1,ndih_constr
          phi0(i)=deg2rad*phi0(i)
          drange(i)=deg2rad*drange(i)
        enddo
        write (iout,*) 'FTORS',ftors
        do i=1,ndih_constr
          ii = idih_constr(i)
          phibound(1,ii) = phi0(i)-drange(i)
          phibound(2,ii) = phi0(i)+drange(i)
        enddo 
      endif
      nnt=1
      nct=nres
      print *,'NNT=',NNT,' NCT=',NCT
      if (itype(1).eq.21) nnt=2
      if (itype(nres).eq.21) nct=nct-1
      if (pdbref) then
        write (iout,'(a,i3)') 'nsup=',nsup
        nstart_seq=nnt
        if (nsup.le.(nct-nnt+1)) then
          do i=0,nct-nnt+1-nsup
            if (seq_comp(itype(nnt+i),itype_pdb(nstart_sup),nsup)) then
              nstart_seq=nnt+i
              goto 111
            endif
          enddo
          write (iout,'(a)') 
     &            'Error - sequences to be superposed do not match.'
          stop
        else
          do i=0,nsup-(nct-nnt+1)
            if (seq_comp(itype(nnt),itype_pdb(nstart_sup+i),nct-nnt+1)) 
     &      then
              nstart_sup=nstart_sup+i
              nsup=nct-nnt+1
              goto 111
            endif
          enddo 
          write (iout,'(a)') 
     &            'Error - sequences to be superposed do not match.'
        endif
  111   continue
        write (iout,*) 'nsup=',nsup,' nstart_sup=',nstart_sup,
     &                 ' nstart_seq=',nstart_seq
      endif
      call init_int_table
      if (refstr) then
        if (.not.pdbref) then
          call read_angles(inp,*38)
          goto 39
   38     write (iout,'(a)') 'Error reading reference structure.'
#ifdef MPL
          call mp_stopall(Error_Msg)
#else
          stop 'Error reading reference structure'
#endif
   39     call chainbuild
          nstart_sup=nnt
          nstart_seq=nnt
          nsup=nct-nnt+1
          do i=1,2*nres
            do j=1,3
              cref(j,i)=c(j,i)
            enddo
          enddo
        endif
        call contact(.true.,ncont_ref,icont_ref)
      endif
      if (indpdb.eq.0 .and. modecalc.ne.2 .and. modecalc.ne.4) then
C If input structure hasn't been supplied from the PDB file read or generate
C initial geometry.
        if (iranconf.eq.0) then
          write (iout,'(a)') 'Initial geometry will be read in.'
          call read_angles(inp,*36)
          goto 37
   36     write (iout,'(a)') 'Error reading angle file.'
#ifdef MPL
	  call mp_stopall(error_msg)
#else
          stop 'Error reading angle file.'
#endif
   37     continue 
        else
          write (iout,'(a)') 'Random-generated initial geometry.'
#ifdef MP
          print *,'MyID',MyID,' MasterID',MasterID
          if (MyID.eq.MasterID) then  
            do itrial=1,3
              itmp=1
              call gen_rand_conf(itmp,*30)
              goto 40
   30         write (iout,*) 'Failed to generate random conformation',
     &          ', itrial=',itrial
              write (*,*) 'Processor:',MyID,
     &          ' Failed to generate random conformation',
     &          ' itrial=',itrial
              call intout

#ifdef LINUX
              call flush(iout)
#else
              call flush_(iout)
#endif
            enddo
            write (iout,'(a,i3,a)') 'Processor:',MyID,
     &        ' error in generating random conformation.'
            write (*,'(a,i3,a)') 'Processor:',MyID,
     &        ' error in generating random conformation.'
#ifdef MPI
            call mpi_abort(mpi_comm_world,error_msg,ierrcode)            
#else
	    call mp_stopall(error_msg)
#endif         
   40       continue
#ifdef MPL
            if (fgProcs.gt.1) then
              msglen=maxres*4*8
              call mp_bcast(theta(1),msglen,BossID,fgGroupID)
              if (MyID.ne.BossID) call chainbuild
            endif
#endif
          endif
#else
          do itrial=1,100
            itmp=1
            call gen_rand_conf(itmp,*30)
            goto 40
   30       write (iout,*) 'Failed to generate random conformation',
     &        ', itrial=',itrial
            write (*,*) 'Failed to generate random conformation',
     &        ', itrial=',itrial
          enddo
          write (iout,'(a,i3,a)') 'Processor:',MyID,
     &      ' error in generating random conformation.'
          write (*,'(a,i3,a)') 'Processor:',MyID,
     &      ' error in generating random conformation.'
          stop
   40     continue
#endif
        endif
      elseif (modecalc.eq.4) then
        read (inp,'(a)') intinname
        open (intin,file=intinname,status='old',err=333)
        write (iout,'(a)') 'intinname',intinname
        write (*,'(a)') 'intinname',intinname
        goto 334
  333   write (iout,'(2a)') 'Error opening angle file ',intinname
#ifdef MPL 
        call mp_stopall(Error_Msg)
#else
        stop 'Error opening angle file.' 
#endif   
  334   continue
      endif
C Generate distance constraints, if the PDB structure is to be regularized. 
      if (nthread.gt.0) then
        call read_threadbase
      endif
      call setup_var
      call intout
      if (ns.gt.0) then
        write (iout,'(/a,i3,a)') 
     &  'The chain contains',ns,' disulfide-bridging cysteines.'
        write (iout,'(20i4)') (iss(i),i=1,ns)
        write (iout,'(/a/)') 'Pre-formed links are:' 
	do i=1,nss
	  i1=ihpb(i)-nres
	  i2=jhpb(i)-nres
	  it1=itype(i1)
	  it2=itype(i2)
	  write (iout,'(2a,i3,3a,i3,a,3f10.3)')
     &    restyp(it1),'(',i1,') -- ',restyp(it2),'(',i2,')',dhpb(i),
     &    ebr,forcon(i)
	enddo
	write (iout,'(a)')
      endif
      call geom_to_var(nvar,x)
cd    write (iout,'(2(a,i3))') 'NNT',NNT,' NCT',NCT
cd    write (iout,'(a)') 'Variable list:'
cd    write (iout,'(i4,f10.5)') (i,rad2deg*x(i),i=1,nvar)
#ifdef MPxL
      write (iout,'(//80(1h*)/20x,a,i4,a/80(1h*)//)') 
     &  'Processor',MyId,': end reading molecular data.'
#endif
      return
      end
c-----------------------------------------------------------------------------
      logical function seq_comp(itypea,itypeb,length)
      implicit none
      integer length,itypea(length),itypeb(length)
      integer i
      do i=1,length
        if (itypea(i).ne.itypeb(i)) then
          seq_comp=.false.
          return
        endif
      enddo
      seq_comp=.true.
      return
      end
c-----------------------------------------------------------------------------
      subroutine read_bridge
C Read information about disulfide bridges.
      include 'DIMENSIONS'
#ifdef MP
      include 'mpif.h'
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
#ifdef MPL
      include 'COMMON.INFO'
#endif
C Read bridging residues.
      read (inp,*) ns,(iss(i),i=1,ns)
      print *,'ns=',ns
      write (iout,*) 'ns=',ns,' iss:',(iss(i),i=1,ns)
C Check whether the specified bridging residues are cystines.
      do i=1,ns
	if (itype(iss(i)).ne.1) then
	  write (iout,'(2a,i3,a)') 
     &   'Do you REALLY think that the residue ',restyp(iss(i)),i,
     &   ' can form a disulfide bridge?!!!'
	  write (*,'(2a,i3,a)') 
     &   'Do you REALLY think that the residue ',restyp(iss(i)),i,
     &   ' can form a disulfide bridge?!!!'
#ifdef MPL
	 call mp_stopall(error_msg)
#else
         stop
#endif
        endif
      enddo
C Read preformed bridges.
      if (ns.gt.0) then
      read (inp,*) nss,(ihpb(i),jhpb(i),i=1,nss)
      write (iout,*) 'nss=',nss,' ihpb,jhpb: ',(ihpb(i),jhpb(i),i=1,nss)
      if (nss.gt.0) then
        nhpb=nss
C Check if the residues involved in bridges are in the specified list of
C bridging residues.
        do i=1,nss
          do j=1,i-1
	    if (ihpb(i).eq.ihpb(j).or.ihpb(i).eq.jhpb(j)
     &      .or.jhpb(i).eq.ihpb(j).or.jhpb(i).eq.jhpb(j)) then
	      write (iout,'(a,i3,a)') 'Disulfide pair',i,
     &      ' contains residues present in other pairs.'
	      write (*,'(a,i3,a)') 'Disulfide pair',i,
     &      ' contains residues present in other pairs.'
#ifdef MPL
	      call mp_stopall(error_msg)
#else
              stop 
#endif
	    endif
          enddo
	  do j=1,ns
	    if (ihpb(i).eq.iss(j)) goto 10
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   10     continue
	  do j=1,ns
	    if (jhpb(i).eq.iss(j)) goto 20
          enddo
          write (iout,'(a,i3,a)') 'Pair',i,' contains unknown cystine.'
   20     continue
          dhpb(i)=dbr
          forcon(i)=fbr
        enddo
        do i=1,nss
          ihpb(i)=ihpb(i)+nres
          jhpb(i)=jhpb(i)+nres
        enddo
      endif
      endif
      return
      end
c----------------------------------------------------------------------------
      subroutine read_threadbase
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
C Read pattern database for threading.
      read (icbase,*) nseq
      do i=1,nseq
        read (icbase,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),
     &   nres_base(2,i),nres_base(3,i)
        read (icbase,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,
     &   nres_base(1,i))
c       write (iout,'(i5,2x,a8,2i4)') nres_base(1,i),str_nam(i),
c    &   nres_base(2,i),nres_base(3,i)
c       write (iout,'(9f8.3)') ((cart_base(k,j,i),k=1,3),j=1,
c    &   nres_base(1,i))
      enddo
      close (icbase)
      if (weidis.eq.0.0D0) weidis=0.1D0
      do i=nnt,nct
        do j=i+2,nct
          nhpb=nhpb+1
          ihpb(nhpb)=i
          jhpb(nhpb)=j
          forcon(nhpb)=weidis
        enddo
      enddo 
      read (inp,*) nexcl,(iexam(1,i),iexam(2,i),i=1,nexcl)
      write (iout,'(a,i5)') 'nexcl: ',nexcl
      write (iout,'(2i5)') (iexam(1,i),iexam(2,i),i=1,nexcl)
      return
      end
c------------------------------------------------------------------------------
      subroutine setup_var
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
C Set up variable list.
      ntheta=nres-2
      nphi=nres-3
      nvar=ntheta+nphi
      nside=0
      do i=2,nres-1
        if (itype(i).ne.10) then
	  nside=nside+1
          ialph(i,1)=nvar+nside
	  ialph(nside,2)=i
        endif
      enddo
      if (indphi.gt.0) then
        nvar=nphi
      else if (indback.gt.0) then
        nvar=nphi+ntheta
      else
        nvar=nvar+2*nside
      endif
cd    write (iout,'(3i4)') (i,ialph(i,1),ialph(i,2),i=2,nres-1)
      return
      end
c----------------------------------------------------------------------------
      subroutine gen_dist_constr
C Generate CA distance constraints.
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.DBASE'
      include 'COMMON.THREAD'
      include 'COMMON.TIME1'
      character*2 iden
      print *,'gen_dist_constr: nnt=',nnt,' nct=',nct
      do i=nnt,nct
        do j=i+2,nct
          nhpb=nhpb+1
          ihpb(nhpb)=i
          jhpb(nhpb)=j
          forcon(nhpb)=weidis
          dhpb(nhpb)=dist(i,j)
        enddo
      enddo 
      write (iout,'(a)') 'Distance constraints:' 
      do i=nss+1,nhpb
        ii=ihpb(i)
        jj=jhpb(i)
        iden='CA'
        if (ii.gt.nres) then
          iden='SC'
          ii=ii-nres
          jj=jj-nres
        endif
        write (iout,'(a,1x,a,i4,3x,a,1x,a,i4,2f8.3)') 
     &  restyp(itype(ii)),iden,ii,restyp(itype(jj)),iden,jj,
     &  dhpb(i),forcon(i)
      enddo
      return
      end
c----------------------------------------------------------------------------
      subroutine map_read
      include 'DIMENSIONS'
      include 'COMMON.MAP'
      include 'COMMON.IOUNITS'
      character*3 angid(4) /'THE','PHI','ALP','OME'/
      character*80 mapcard,ucase
      do imap=1,nmap
        read (inp,'(a)') mapcard
        mapcard=ucase(mapcard)
        if (index(mapcard,'PHI').gt.0) then
          kang(imap)=1
        else if (index(mapcard,'THE').gt.0) then
          kang(imap)=2
        else if (index(mapcard,'ALP').gt.0) then
          kang(imap)=3
        else if (index(mapcard,'OME').gt.0) then
          kang(imap)=4
        else
          write(iout,'(a)')'Error - illegal variable spec in MAP card.'
          stop 'Error - illegal variable spec in MAP card.'
        endif
        call readi (mapcard,'RES1',res1(imap),0)
        call readi (mapcard,'RES2',res2(imap),0)
        if (res1(imap).eq.0) then
          res1(imap)=res2(imap)
        else if (res2(imap).eq.0) then
          res2(imap)=res1(imap)
        endif
        if(res1(imap)*res2(imap).eq.0 .or. res1(imap).gt.res2(imap))then
          write (iout,'(a)') 
     &    'Error - illegal definition of variable group in MAP.'
          stop 'Error - illegal definition of variable group in MAP.'
        endif
        call reada(mapcard,'FROM',ang_from(imap),0.0D0)
        call reada(mapcard,'TO',ang_to(imap),0.0D0)
        call readi(mapcard,'NSTEP',nstep(imap),0)
        if (ang_from(imap).eq.ang_to(imap) .or. nstep(imap).eq.0) then
          write (iout,'(a)') 
     &     'Illegal boundary and/or step size specification in MAP.'
          stop 'Illegal boundary and/or step size specification in MAP.'
        endif
      enddo ! imap
      return
      end 
c----------------------------------------------------------------------------
      subroutine mcmread
      include 'DIMENSIONS'
      include 'COMMON.MCM'
      include 'COMMON.MCE'
      include 'COMMON.IOUNITS'
      character*80 ucase
      character*320 mcmcard
      call card_concat(mcmcard)
      call readi(mcmcard,'MAXACC',maxacc,100)
      call readi(mcmcard,'MAX_MCM_IT',max_mcm_it,10000)
      call readi(mcmcard,'MAXTRIAL',maxtrial,100)
      call readi(mcmcard,'MAXTRIAL_ITER',maxtrial_iter,1000)
      call readi(mcmcard,'MAXREPM',maxrepm,200)
      call reada(mcmcard,'RANFRACT',RanFract,0.5D0)
      call reada(mcmcard,'POOL_FRACT',pool_fraction,0.01D0)
      call reada(mcmcard,'OVERLAP',overlap_cut,1.0D3)
      call reada(mcmcard,'E_UP',e_up,5.0D0)
      call reada(mcmcard,'DELTE',delte,0.1D0)
      call readi(mcmcard,'NSWEEP',nsweep,5)
      call readi(mcmcard,'NSTEPH',nsteph,0)
      call readi(mcmcard,'NSTEPC',nstepc,0)
      call reada(mcmcard,'TMIN',tmin,298.0D0)
      call reada(mcmcard,'TMAX',tmax,298.0D0)
      call readi(mcmcard,'NWINDOW',nwindow,0)
      call readi(mcmcard,'PRINT_MC',print_mc,0)
      print_stat=(index(mcmcard,'NO_PRINT_STAT').le.0)
      print_int=(index(mcmcard,'NO_PRINT_INT').le.0)
      ent_read=(index(mcmcard,'ENT_READ').gt.0)
      call readi(mcmcard,'SAVE_FREQ',save_frequency,1000)
      call readi(mcmcard,'MESSAGE_FREQ',message_frequency,1000)
      call readi(mcmcard,'POOL_READ_FREQ',pool_read_freq,5000)
      call readi(mcmcard,'POOL_SAVE_FREQ',pool_save_freq,1000)
      call readi(mcmcard,'PRINT_FREQ',print_freq,1000)
      if (nwindow.gt.0) then
        read (inp,*) (winstart(i),winend(i),i=1,nwindow)
        do i=1,nwindow
          winlen(i)=winend(i)-winstart(i)+1
        enddo
      endif
      if (tmax.lt.tmin) tmax=tmin
      if (tmax.eq.tmin) then
        nstepc=0
        nsteph=0
      endif
      if (nstepc.gt.0 .and. nsteph.gt.0) then
        tsteph=(tmax/tmin)**(1.0D0/(nsteph+0.0D0)) 
        tstepc=(tmax/tmin)**(1.0D0/(nstepc+0.0D0)) 
      endif
C Probabilities of different move types
      sumpro_type(0)=0.0D0
      call reada(mcmcard,'MULTI_BOND',sumpro_type(1),1.0d0)
      call reada(mcmcard,'ONE_ANGLE' ,sumpro_type(2),2.0d0)
      sumpro_type(2)=sumpro_type(1)+sumpro_type(2)
      call reada(mcmcard,'THETA'     ,sumpro_type(3),0.0d0)
      sumpro_type(3)=sumpro_type(2)+sumpro_type(3)
      call reada(mcmcard,'SIDE_CHAIN',sumpro_type(4),0.5d0)
      sumpro_type(4)=sumpro_type(3)+sumpro_type(4)
      do i=1,MaxMoveType
        print *,'i',i,' sumprotype',sumpro_type(i)
        sumpro_type(i)=sumpro_type(i)/sumpro_type(MaxMoveType)
        print *,'i',i,' sumprotype',sumpro_type(i)
      enddo
      return
      end 
c----------------------------------------------------------------------------
      subroutine read_minim
      include 'DIMENSIONS'
      include 'COMMON.MINIM'
      include 'COMMON.IOUNITS'
      character*80 ucase
      character*320 minimcard
      call card_concat(minimcard)
      call readi(minimcard,'MAXMIN',maxmin,2000)
      call readi(minimcard,'MAXFUN',maxfun,5000)
      call reada(minimcard,'TOLF',tolf,1.0D-2)
      call reada(minimcard,'RTOLF',rtolf,1.0D-4)
      write (iout,'(/80(1h*)/20x,a/80(1h*))') 
     &         'Options in energy minimization:'
      write (iout,'(a,i5,a,i5,a,1pe14.5,a,1pe14.5)')
     & 'MaxMin:',MaxMin,' MaxFun:',MaxFun,' TolF:',TolF,' RTolF:',RTolF
      return
      end
c----------------------------------------------------------------------------
      subroutine read_angles(kanal,*)
      include 'DIMENSIONS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      read (kanal,*,err=10,end=10) (theta(i),i=3,nres)
      read (kanal,*,err=10,end=10) (phi(i),i=4,nres)
      read (kanal,*,err=10,end=10) (alph(i),i=2,nres-1)
      read (kanal,*,err=10,end=10) (omeg(i),i=2,nres-1)
      do i=1,nres
        theta(i)=deg2rad*theta(i)
        phi(i)=deg2rad*phi(i)
        alph(i)=deg2rad*alph(i)
        omeg(i)=deg2rad*omeg(i)
      enddo
      return
   10 return1
      end
c----------------------------------------------------------------------------
      subroutine reada(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      double precision wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*) wartosc
      return
      end
c----------------------------------------------------------------------------
      subroutine readi(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      integer wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*) wartosc
      return
      end
c----------------------------------------------------------------------------
      subroutine openunits
      include 'DIMENSIONS'    
#ifdef MP
      include 'mpif.h'
#endif
      include 'COMMON.IOUNITS'
#ifdef MP
      include 'COMMON.INFO'
      character*3 liczba
#endif
      integer lenpre,lenpot,ilen
      external ilen
      call getenv('PREFIX',prefix)
      call getenv('POT',pot)
      lenpre=ilen(prefix)
      lenpot=ilen(pot)
      entname=prefix(:lenpre)//'_'//pot(:lenpot)//'.entr'
C Get the names and open the input files

      open (1,file=prefix(:ilen(prefix))//'.inp',status='old')
      open (9,file=prefix(:ilen(prefix))//'.intin',status='unknown')
      open (18,file=prefix(:ilen(prefix))//'.entin',status='unknown')
C Get parameter filenames and open the parameter files.
      call getenv('THETPAR',thetname)
      open (ithep,file=thetname,status='old')
      call getenv('ROTPAR',rotname)
      open (irotam,file=rotname,status='old')
      call getenv('TORPAR',torname)
      open (itorp,file=torname,status='old')
      call getenv('ELEPAR',elename)
      open (ielep,file=elename,status='old')
      call getenv('SIDEPAR',sidename)
      open (isidep,file=sidename,status='old')
      call getenv('PATTERN',patname)
      open (icbase,file=patname,status='old')
      print '(2a)','thetname=',thetname,'rotname=',rotname,
     &  'torname=',torname,'elename=',elename,'sidename=',sidename,
     &  'patname=',patname
#ifdef MP
      if (MyID.eq.BossID) then
      MyRank = MyID/fgProcs
      print *,'OpenUnits: processor',MyID,' MyRank=',MyRank
      call numstr(MyRank,liczba)
      outname=prefix(:lenpre)//'.out_'//pot(:lenpot)//liczba
      intname=prefix(:lenpre)//'_'//pot(:lenpot)//liczba//'.int'
      pdbname=prefix(:lenpre)//'_'//pot(:lenpot)//liczba//'.pdb'
      mol2name=prefix(:lenpre)//'_'//pot(:lenpot)//liczba//'.mol2'
      statname=prefix(:lenpre)//'_'//pot(:lenpot)//liczba//'.stat'
#else
      outname=prefix(:lenpre)//'.out_'//pot(:lenpot)
      intname=prefix(:lenpre)//'_'//pot(:lenpot)//'.int'
      pdbname=prefix(:lenpre)//'_'//pot(:lenpot)//'.pdb'
      mol2name=prefix(:lenpre)//'_'//pot(:lenpot)//'.mol2'
      statname=prefix(:lenpre)//'_'//pot(:lenpot)//'.stat'
#endif
#ifdef AIX
      open(iout,file=outname,status='unknown')
      open(igeom,file=intname,status='unknown',position='append')
      open(ipdb,file=pdbname,status='unknown')
      open(imol2,file=mol2name,status='unknown')
      open(istat,file=statname,status='unknown',position='append')
#else
      open(iout,file=outname,status='unknown')
      open(igeom,file=intname,status='unknown',access='append')
      open(ipdb,file=pdbname,status='unknown')
      open(imol2,file=mol2name,status='unknown')
      open(istat,file=statname,status='unknown',access='append')
#endif
#ifdef MP
      endif ! (MyID.eq.BossID)
#endif
      return
      end
c----------------------------------------------------------------------------
      subroutine card_concat(card)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      character*(*) card
      character*80 karta,ucase
      external ilen
      read (inp,'(a)') karta
      karta=ucase(karta)
      card=' '
      do while (karta(80:80).eq.'&')
        card=card(:ilen(card)+1)//karta(:79)
        read (inp,'(a)') karta
        karta=ucase(karta)
      enddo
      card=card(:ilen(card)+1)//karta
      return
      end
