CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                              C
C                                U N R E S                                     C
C                                                                              C
C Program to carry out conformational search of proteins in an united-residue  C
C approximation.                                                               C
C                                                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
#endif
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
#ifdef MP
      include 'COMMON.INFO'
#endif
#ifdef MPI
      include 'COMMON.SETUP'
#endif
      double precision varia(maxvar)
      double precision hrtime,mintime,sectime
      double precision przes(3),obrot(3,3)
      double precision enetb1(max_threadss),coord1(maxvar,max_threadss)
      logical non_conv
      character*64 text_mode_calc(-1:8) /'SC rotamer distribution',
     & 'Energy evaluation or minimization',
     & 'Regularization of PDB structure',
     & 'Threading of a sequence on PDB structures',
     & 'Monte Carlo (with minimization) ',
     & 'Energy minimization of multiple conformations',
     & 'Checking energy gradient',
     & 'Entropic sampling Monte Carlo (with minimization)',
     & 'Energy map',
     & 'CSA calculations'/
      common /srutu/ icall
      logical eof
      external ilen
      integer iparent1(2*max_threadss,2*max_threadss),
     & nparent2(max_threadss)
#ifdef MP
C Initialize task.
      call init_task
#else
      call initialize
      call openunits
      write (iout,'(80(1h*)/a/80(1h*))') 
     & 'United-residue force field calculation - serial job.'
#endif
      call parmread
      call read_control
      if (minim) call read_minim
      if (modecalc.eq.3 .or. modecalc.eq.6) call mcmread
      call molread
#ifdef MPL
      msglen_var=8*(nvar+3)
#endif
      write (iout,'(2a/)') 
     & text_mode_calc(modecalc)(:ilen(text_mode_calc(modecalc))),
     & ' calculation.' 
      if (minim) write (iout,'(a)') 
     &  'Conformations will be energy-minimized.'
      write (iout,'(80(1h*)/)') 
cd    print *,'MAIN: nnt=',nnt,' nct=',nct,' modecalc=',modecalc
      if (modecalc.eq.-1) then
        call check_sc_distr
      else if (modecalc.eq.0) then
        call hpb_partition
        call chainbuild
        icall=1
        call etotal(evdw,evdw1,evdw2,ees,ebe,esc,etors,ehpb,ecorr,etot)
        call enerprint(evdw,evdw1,evdw2,ees,ebe,esc,etors,ehpb,ecorr,
     &                 etot)
c         call mol2out(etot,titel(:32))
        if (minim) then
          call geom_to_var(nvar,varia)
          print *,'Calling MINIMIZE.'
#ifdef MPL
          IF (MyID.eq.BossID) THEN
#endif
          call minimize(etot,varia,iretcode,nfun)
          print *,'SUMSL return code is',iretcode
          print *,'refstr=',refstr
          if (refstr) then
            call fitsq(rms,c(1,nstart_seq),cref(1,nstart_sup),
     &                 nsup,przes,obrot,non_conv)
            rms=dsqrt(rms)
            call contact(.false.,ncont,icont)
            frac=contact_fract(ncont,ncont_ref,icont,icont_ref)
            write (iout,'(/a,f7.3,a,f7.1)')
     &      'RMS deviation from the reference structure:',rms,
     &      ' % of native contacts:',frac*100
          endif ! refstr

#ifdef MPL
          call free_slaves(nfun,varia)
          
          ELSE
          
          call ergastulum

          ENDIF ! (MyID.eq.BossID)
cd        write (*,*) 'Processor',MyID,' back in MAIN.'
#endif
cd        write (*,'(a,i5,4(/10f8.3))') 'Processor',MyID,' variables:',
cd   &    (phi(i),i=4,nres),(theta(i),i=2,nres),(alph(i),i=2,nres-1),
cd   &    (omeg(i),i=2,nres-1)
          call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,ecorr,
     &                etot)
#ifdef MPL
          IF (MyID.eq.BossID) THEN
#endif
          call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                     ecorr,etot)
          call intout
          call briefout(0,etot)
          if (refstr) then
            call contact(.true.,ncont,icont)
            frac=contact_fract(ncont,ncont_ref,icont,icont_ref)
            write (iout,'(a,f10.5)') 'Fraction of native contacts:',frac
          endif
          write (iout,'(a,i3)') 'SUMSL return code:',iretcode
          write (iout,'(a,i20)') '# of energy evaluations:',nfun+1
#ifdef MPL
          ENDIF ! (MyID.eq.BossID)
#endif
        endif
#ifdef MPL
        do i=0,NProcs-1
          Koniec(i)=.true.
        enddo
        if (MyID.eq.BossID .and. outpdb) then
          call pdbout(etot,titel(:32))
        endif
        if (MyID.eq.BossID .and. outmol2) then
          call mol2out(etot,titel(:32))
        endif
#else
        if (outpdb) call pdbout(etot,titel(:32))
        if (outmol2) call mol2out(etot,titel(:32))
#endif
      else if (modecalc.eq.1) then
        call gen_dist_constr
        call hpb_partition
        call sc_conf
        call intout
#ifdef MPL
        IF (MyID.eq.BossID) THEN
#endif
        call regularize(nct-nnt+1,etot,rms,cref(1,nnt),iretcode)
#ifdef MPL
        call free_slaves(1,varia)
         
        ELSE
          
        call ergastulum

        ENDIF ! (MyID.eq.BossID)
#endif
        call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,ecorr,
     &              etot)
#ifdef MPL
        IF (MyID.eq.BossID) THEN
#endif
        etot=etot-ehpb
        call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                     ecorr,etot)
        call intout
        call briefout(0,etot)
        if (outpdb) call pdbout(etot,titel(:32))
        if (outmol2) call mol2out(etot,titel(:32))
        if (refstr) then
          call contact(.true.,ncont,icont)
          frac=contact_fract(ncont,ncont_ref,icont,icont_ref)
          write (iout,'(a,f10.5)') 'Fraction of native contacts:',frac
        endif
        write (iout,'(a,i3)') 'SUMSL return code:',iretcode
#ifdef MPL
        ENDIF ! (MyID.eq.MasterID)
        do i=0,NProcs-1
          Koniec(i)=.true.
        enddo
#endif
      else if (modecalc.eq.2) then
        call hpb_partition
#ifdef MPL
        IF (MyID.eq.BossID) THEN
#endif
        call thread_seq
#ifdef MPL
        call free_slaves(1,varia)
          
        ELSE
          
        call ergastulum

        ENDIF ! (MyID.eq.BossID)
#endif
      else if (modecalc.eq.3 .or. modecalc .eq.6) then
        call hpb_partition
#ifdef MPL
        IF (MyID.eq.BossID) THEN
#endif
        call mcm_setup
        if (minim) then
          if (modecalc.eq.3) then
            call do_mcm(n_accum,coord1,enetb1,nparent1,iparent1)
          else
            call entmcm
          endif
        else
          call monte_carlo
        endif
#ifdef MPL
        call free_slaves(1,varia)
          
        ELSE
          
        call ergastulum

        ENDIF ! (MyID.eq.BossID)
#endif
#ifdef MPL
        do i=0,NProcs-1
          Koniec(i)=.true.
        enddo
#endif
      elseif (modecalc.eq.4) then
        call hpb_partition
        eof=.false.
        do while (.not. eof)
          read (intin,*,end=11,err=11) iconf
          call read_angles(intin,*11)  
          call geom_to_var(nvar,varia)
          write (iout,'(a,i7)') 'Conformation #',iconf
          if (minim) call minimize(etot,varia,iretcode,nfun)
          call etotal(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,ecorr,
     &                etot)
          call enerprint(evdw,evdw1,evdw2,ees,ebe,escloc,etors,ehpb,
     &                     ecorr,etot)
          if (minim) call briefout(it,etot) 
        enddo
   11   continue
#ifdef MPL
        do i=0,NProcs-1
          Koniec(i)=.true.
        enddo
#endif
      else if (modecalc.eq.5) then
        call hpb_partition
        call chainbuild
        icall=1
        call etotal(evdw,evdw1,evdw2,ees,ebe,esc,etors,ehpb,ecorr,etot)
        call enerprint(evdw,evdw1,evdw2,ees,ebe,esc,etors,ehpb,ecorr,
     &                 etot)
        print *,'icheckgrad=',icheckgrad
#ifdef MPL
        do i=0,nprocs-1
          koniec(i)=.true.
        enddo
#endif
        goto (10,20,30) icheckgrad
  10    call check_ecart
        goto 40
  20    call check_cartgrad
        goto 40
  30    call check_eint
      else if (ModeCalc.eq.7) then
C Mapping
        call hpb_partition
        call map_read
        call map
#ifdef MPI
      else if (ModeCalc.eq.8) then
C Conformational Space Annealling programmed by Jooyoung Lee.
C This method works only with parallel machines!
        call hpb_partition
        call together
      else
#endif
        write (iout,'(a)') 'Unknown ModeCalc=',ModeCalc
      endif
  40  continue
#ifdef MP
C Finish task.
      write (iout,'(a,i4,a)') 'Processor',MyId,' is finishing work.'
      write (*,'(a,i4,a)') 'Processor',MyId,' is finishing work.'
      call finish_task
C Cleaning up...
      stop 'Bye Bye...'
#else
      call dajczas(tcpu(),hrtime,mintime,sectime)
      stop '********** Program terminated normally.'
#endif
      end
